# -*- coding: utf-8 -*-

from .device import Device
from .utils import notNone
from .const import (
    #LOGGER,
    STATUS_RESPONSE_METERS,
    STATUS_RESPONSE_EMETERS,
    STATUS_RESPONSE_METERS_POWER,
    STATUS_RESPONSE_METERS_TOTAL,
    STATUS_RESPONSE_METERS_VOLTAGE,
    STATUS_RESPONSE_METERS_PF,
    STATUS_RESPONSE_METERS_CURRENT,
    STATUS_RESPONSE_METERS_TOTAL_RETURNED,
    INFO_VALUE_CURRENT_CONSUMPTION,
    INFO_VALUE_TOTAL_CONSUMPTION,
    INFO_VALUE_TOTAL_RETURNED,
    INFO_VALUE_VOLTAGE,
    INFO_VALUE_POWER_FACTOR,
    INFO_VALUE_CURRENT,
    ATTR_POS,
    ATTR_PATH,
    ATTR_FMT
)

class PowerMeter(Device):
    """Class to represent a power meter value"""
    def __init__(self, block, channel, position = None,
                 tot_pos = None, voltage_to_block=False, em=False):
        #Todo: voltage_to_block
        super(PowerMeter, self).__init__(block)
        self.id = block.id
        if channel > 0:
            self.id += "-" + str(channel)
            self._channel = channel - 1
            self.device_nr = channel
        else:
            self._channel = 0
        self.is_device = False
        self.is_sensor = True

        # if meters is None:
        #     self.meters = [self._channel]
        # else:
        #     self.meters = meters
        #self.sensor_values = {}
        self.device_type = "POWERMETER"
        self.info_values = {}
        self.state = None
        meters = "emeters" if em else "meters"
        self._state_cfg = {
            ATTR_POS: notNone(position, [111, 4101, 4102, 4105]),
            ATTR_PATH: meters + '/$/power',
            ATTR_FMT: ['float']
        }
        divider = None if em else '/60'
        self._info_value_cfg = {
            INFO_VALUE_POWER_FACTOR : {
                ATTR_POS: [114, 4110],
                ATTR_PATH: meters + 'eters/$/pf',
                ATTR_FMT: ['float']
            },
            INFO_VALUE_CURRENT : {
                ATTR_POS: [115, 4109],
                ATTR_PATH: meters + '/$/current',
                ATTR_FMT: ['float']
            },
            INFO_VALUE_TOTAL_CONSUMPTION : {
                ATTR_POS: tot_pos or [4103, 4104, 4106],
                ATTR_PATH: meters + '/$/total',
                ATTR_FMT: ['float', divider, 'round:2']
            },
            INFO_VALUE_TOTAL_RETURNED :
            {
                ATTR_POS: [4107],
                ATTR_PATH: meters + '/$/total_returned',
                ATTR_FMT: ['float', divider, 'round:2']
            }
        }
        (self.block if voltage_to_block else self) \
            ._info_value_cfg[INFO_VALUE_VOLTAGE] = \
            {
                ATTR_POS: [116, 4108],
                ATTR_PATH: meters + '/$/voltage',
                ATTR_FMT: ['float']
            }

    # """
    # def update_status_information(self, status):
    #     ""Update the status information.""
    #      factor = 1
    #     if STATUS_RESPONSE_EMETERS in status:
    #         meters = status.get(STATUS_RESPONSE_EMETERS) #Shelly EM
    #     else:
    #         meters = status.get(STATUS_RESPONSE_METERS)
    #         factor = 60
    #     if meters:
    #         power = 0
    #         total = None
    #         total_returned = None
    #         for meterpos in self.meters:
    #             meter = meters[meterpos]
    #             if meter.get(STATUS_RESPONSE_METERS_POWER) is not None:
    #                 power += float(meter.get(STATUS_RESPONSE_METERS_POWER))
    #             if meter.get(STATUS_RESPONSE_METERS_TOTAL) is not None:
    #                 total = (total or 0) + \
    #                     float(meter.get(STATUS_RESPONSE_METERS_TOTAL))
    #             if meter.get(STATUS_RESPONSE_METERS_TOTAL_RETURNED) is not None:
    #                 total_returned = (total_returned or 0) + \
    #                     float(meter.get(STATUS_RESPONSE_METERS_TOTAL_RETURNED))
    #             if meter.get(STATUS_RESPONSE_METERS_VOLTAGE) is not None:
    #                 if self._voltage_to_block:
    #                     self.block.info_values[INFO_VALUE_VOLTAGE] = \
    #                         float(meter.get(STATUS_RESPONSE_METERS_VOLTAGE))
    #                 else:
    #                     self.info_values[INFO_VALUE_VOLTAGE] = \
    #                         float(meter.get(STATUS_RESPONSE_METERS_VOLTAGE))
    #             elif status.get(STATUS_RESPONSE_METERS_VOLTAGE) is not None:
    #                 #Fix for Shelly 2.5 etc
    #                 self.block.info_values[INFO_VALUE_VOLTAGE] = \
    #                     float(status.get(STATUS_RESPONSE_METERS_VOLTAGE))
    #             if meter.get(STATUS_RESPONSE_METERS_PF) is not None:
    #                 self.info_values[INFO_VALUE_POWER_FACTOR] = \
    #                     float(meter.get(STATUS_RESPONSE_METERS_PF))
    #             if meter.get(STATUS_RESPONSE_METERS_CURRENT) is not None:
    #                 self.info_values[INFO_VALUE_CURRENT] = \
    #                     float(meter.get(STATUS_RESPONSE_METERS_CURRENT))
    #         self.state = power
    #         if total_returned is not None:
    #             self.info_values[INFO_VALUE_TOTAL_RETURNED] \
    #                 = round(total_returned / factor)
    #         if total is not None:
    #             self.info_values[INFO_VALUE_TOTAL_CONSUMPTION] \
    #                 = round(total / factor)
    #         self._update(self.state, info_values=self.info_values) """

    # def update_coap(self, payload):
    #     """Get the power"""
    #     update = False
    #     if self._positions:
    #         self.state = sum(payload.get(pos, 0) for pos in self._positions)
    #         update = True
    #     """ 
    #     if self._volt_pos and self._volt_pos in payload:
    #         update = True
    #         if self._voltage_to_block:
    #             self.block.info_values[INFO_VALUE_VOLTAGE] = \
    #                 round(payload[self._volt_pos], 2)
    #         else:
    #             self.info_values[INFO_VALUE_VOLTAGE] = \
    #                 round(payload[self._volt_pos], 2)
    #     if self._pf_pos and self._pf_pos in payload:
    #         update = True
    #         self.info_values[INFO_VALUE_POWER_FACTOR] = \
    #             round(payload[self._pf_pos], 2)
    #     if self._current_pos and self._current_pos in payload:
    #         update = True
    #         self.info_values[INFO_VALUE_CURRENT] = \
    #             round(payload[self._current_pos], 2)
    #     if self._tot_pos and self._tot_pos in payload:
    #         update = True
    #         self.info_values[INFO_VALUE_TOTAL_CONSUMPTION] = \
    #             round(payload[self._tot_pos]/60)
    #     if update:
    #         self._update(self.state, info_values=self.info_values) """


# -*- coding: utf-8 -*-
# pylint: disable=broad-except, bare-except, invalid-name

import re
import ipaddress
from zeroconf import ( ServiceBrowser, Zeroconf )

MATCH_NAME = \
    re.compile("(?P<devtype>shelly.+)-(?P<id>[0-9A-F]+)._http._tcp.local.")

EXCLUDE = ['shellydw', 'shellyht', 'shellyflood']

class MDns:

    def remove_service(self, _zeroconf, type, name):
        pass

    def add_service(self, zeroconf, type, name):
        test = MATCH_NAME.fullmatch(name)
        if test:
            dev_type = test.group('devtype')
            if dev_type in EXCLUDE:
                return
            info = zeroconf.get_service_info(type, name)
            for addr in info.addresses:
                ipaddr = str(ipaddress.IPv4Address(addr))
                self._root.add_device_by_ip(ipaddr, "mDns")

    def update_service(self, zconf, type, name):
        """ Update a service in the collection. """
        self.add_service(zconf, type, name)

    def __init__(self, root, zeroconf = None):
        self._root = root
        self._common_zeroconf = zeroconf
        self._zeroconf = None
        self._browser = None

    def start(self):
        self._zeroconf = zeroconf = self._common_zeroconf or Zeroconf()
        self._browser = \
            ServiceBrowser(zeroconf, "_http._tcp.local.", self)

    def close(self):
        if self._zeroconf:
            if not self._common_zeroconf:
                self._zeroconf.close()
            self._zeroconf = None



# -*- coding: utf-8 -*-
# pylint: disable=broad-except, bare-except

import json
import time
try:
    import asyncio
except:
    pass
import threading
from datetime import datetime, timedelta

from .compat import s, uc, urlencode
from .const import LOGGER

try:
    import http.client as httplib
except:
    import httplib

class Cloud():
    def __init__(self, root, server, key):
        self.auth_key = key
        self.server = server.replace("https://", "").replace("Server: ","")
        self._last_update = None
        self.update_interval = timedelta(minutes=1)
        self._device_list = None
        self._room_list = None
        self._cloud_thread = None
        self._last_post = datetime.now()
        self._root = root
        self.http_lock = threading.Lock()
        self.stopped = False

    def start(self, cleanCache):
        if cleanCache:
            self._root.save_cache('cloud', {})
        self._cloud_thread = threading.Thread(target=self._update_loop)
        self._cloud_thread.name = "Cloud"
        self._cloud_thread.daemon = True
        self._cloud_thread.start()

    def stop(self):
       self.stopped = True

    def _update_loop(self):
        if self._root.event_loop:
            asyncio.set_event_loop(self._root.event_loop)
        try:
            cloud = self._root.load_cache('cloud')
            if cloud:
                self._device_list = cloud['device_list']
                self._room_list = cloud['room_list']
        except Exception as ex:
            LOGGER.error("Error load cloud cache, %s", ex)
        while not self._root.stopped.isSet() and not self.stopped:
            try:
                if self._last_update is None or \
                    datetime.now() - self._last_update \
                                    > self.update_interval:
                    self._last_update = datetime.now()
                    LOGGER.debug("Update from cloud")
                    devices = self.get_device_list()
                    if devices:
                        self._device_list = devices

                    rooms = self.get_room_list()
                    if rooms:
                        self._room_list = rooms

                    self._root.save_cache('cloud', \
                        {'device_list' : self._device_list,
                         'room_list' : self._room_list}
                    )
                else:
                    self._root.stopped.wait(5)
            except Exception as ex:
                LOGGER.error("Error update cloud, %s", ex)

    def _post(self, path, params=None, retry=0):
        with self.http_lock:
            while datetime.now() - self._last_post < timedelta(seconds=2):
                time.sleep(1)
            self._last_post = datetime.now()

        json_body = None
        params = params or {}
        try:
            LOGGER.debug("POST to Shelly Cloud")
            conn = httplib.HTTPSConnection(self.server, timeout=15)
            headers = {'Content-Type' : 'application/x-www-form-urlencoded',
                        "Connection": "close"}
            params["auth_key"] = self.auth_key
            conn.request("POST", "/" + path, urlencode(params),
                         headers)
            resp = conn.getresponse()

            if resp.status == 200:
                body = resp.read()
                json_body = json.loads(s(body))
            else:
                if retry < 2:
                    return self._post(path, params, retry + 1)
                else:
                    LOGGER.warning("Error receive JSON from cloud, %s : %s", \
                                   resp.reason, resp.read())
        except Exception as ex:
            LOGGER.warning("Error connect cloud, %s", ex)
        finally:
            if conn:
                conn.close()

        return json_body

    def get_device_name(self, _id, idx=None, _ext_sensor=None):
        """Return name using template for device"""
        dev = None
        add_idx = idx and idx > 1
        if idx:
            dev = self._device_list.get(_id + '_' + str(idx-1))
            if dev:
                add_idx = False
        if not dev:
            dev = self._device_list.get(_id)
        if dev:
            name = dev['name']
            if _ext_sensor is not None and 'external_sensors_names' in dev:
                ext_sensors = dev['external_sensors_names']
                if str(_ext_sensor) in ext_sensors:
                    ext_name = ext_sensors[str(_ext_sensor)]['name']
                    if ext_name != 'unnamed':
                        name = ext_name
                        add_idx = False
            room = ""
            try:
                room_id = str(dev['room_id'])
                if room_id == '-10':
                    room = '[Hidden]'
                elif room_id in self._room_list:
                    room = self._room_list[room_id]['name']
                else:
                    room = str(room_id)
            except:
                pass
            tmpl = uc(self._root.tmpl_name)
            value = tmpl.format(id=id, name=name, room=room)
            if add_idx:
                value = value + " - " + str(idx)
            return value
        return None

    def get_relay_usage(self, _id, channel):
        dev_id = (_id + "_" + str(channel) if channel else _id).lower()
        if self._device_list and dev_id in self._device_list:
            dev = self._device_list[dev_id]
            if 'relay_usage' in dev:
                return dev['relay_usage']
        return None

    def get_room_name(self, _id):
        """Return room name of a device"""
        room = None
        if self._device_list and _id in self._device_list:
            dev = self._device_list[_id]
            try:
                room_id = str(dev['room_id'])
                if room_id == '-10':
                    room = '[Hidden]'
                elif room_id in self._room_list:
                    room = self._room_list[room_id]['name']
                else:
                    room = str(room_id)
            except:
                pass
        return room

    def get_device_list(self):
        resp = self._post("interface/device/list")
        return resp['data']['devices'] if resp else None

    def get_status(self):
        self._post("device/all_status")

    def get_room_list(self):
        resp = self._post("interface/room/list")
        return resp['data']['rooms'] if resp else None



import traceback

from datetime import datetime

from .const import (
    BLOCK_INFO_VALUES,
    LOGGER,
    ATTR_PATH,
    ATTR_FMT,
    ATTR_POS,
    ATTR_CHANNEL,
    REGEX_VER,
    SRC_COAP,
    SRC_STATUS
)
class Base(object):

    def __init__(self):
        self.info_values = {}
        self.info_values_updated = {}
        self._info_value_cfg = {}
        self.info_values_status = {}
        self.info_values_coap = {}
        self._state_cfg = None
        self.state_status = None
        self.state_coap = None
        self._channel = 0
        self.cb_updated = []
        self.need_update = False

    def raise_updated(self, force=False):
        if not force and not self.need_update:
            return
        self.need_update = False
        for callback in self.cb_updated:
            callback(self)

    def _fmt_info_value(self, value, cfg, prefix):
        fmt_list = cfg.get(ATTR_FMT, None)
        if type(fmt_list) is dict:
            fmt_list = fmt_list.get(prefix)
        if fmt_list:
            if not type(fmt_list) is list:
                fmt_list = [fmt_list]
            for fmt in fmt_list:
                if fmt is None:
                    continue
                if callable(fmt):
                    value = fmt(value)
                    continue
                params = fmt.split(':')
                cmd = params[0]
                if cmd == 'bool':
                    value = value > 0
                elif cmd == "round":
                    if len(params)>1:
                        value = round(value, int(params[1]))
                    else:
                        value = round(value)
                elif cmd == "float":
                    value = float(value)
                elif cmd[0] == '/':
                    div = int(fmt[1:])
                    value = value / div
                elif cmd == "ver":
                    value = self.parent._firmware_mgr.format(value)
                                                
        return value

    def _get_status_value(self, status, cfg):
        value = status
        path = cfg.get(ATTR_PATH)
        if not path:
            return None
        for key in path.split('/'):
            if value is not None:
                if key == '$':
                    ch = cfg.get(ATTR_CHANNEL, self._channel)
                    value = value[ch]
                else:
                    value = value.get(key, None)
        return value

    def _update_state(self, status, cfg):
        value = self._get_status_value(status, cfg)
        if value is not None:
            value = self._fmt_info_value(value, cfg, SRC_STATUS)
            self._set_state(value, SRC_STATUS)
            if self.lazy_load:
                self.block.parent.callback_add_device(self)

    def _update_info_value(self, name, status, cfg):
        value = self._get_status_value(status, cfg)
        if value is not None:
            value = self._fmt_info_value(value, cfg, SRC_STATUS)
            self.set_info_value(name, value, SRC_STATUS)
            # if self.info_values.get(name) != value:
            #     self.info_values[name] = value
            #     self.need_update = True
            # self.info_values_updated[name] = datetime.now()
            # self.info_values_status[name] = value

    def _get_coap_value(self, cfg, payload):
        if ATTR_POS in cfg:
            pos_list = cfg[ATTR_POS]
            if not type(pos_list) is list:
                pos_list = [pos_list]
            for _pos in pos_list:
                ch = cfg.get(ATTR_CHANNEL, self._channel)
                if _pos < 1000:
                    pos = _pos + 10 * ch
                else:
                    pos = _pos + 100 * ch
                if pos in payload:
                    value = payload.get(pos)
                    value = self._fmt_info_value(value, cfg, SRC_COAP)
                    return value
        return None

    def __update_info_values_coap(self, payload, cfg):
        for name, cfg in cfg.items():
            value = self._get_coap_value(cfg, payload)
            self.set_info_value(name, value, SRC_COAP)
            # if value is not None:
            #     self.info_values_updated[name] = datetime.now()
            #     self.info_values_coap[name] = value
            #     if self.info_values.get(name) != value:
            #         self.info_values[name] = value
            #         self.need_update = True

    def _set_state(self, new_state, src):
        if not new_state is None:
            if src == SRC_COAP:
                self.state_coap = new_state
            if src == SRC_STATUS:
                self.state_status = new_state
            if self.state != new_state:
                self.state = new_state
                self.need_update = True

    def _update_info_values_coap(self, payload, extra_info_value_cfg=None):
        if self._state_cfg:
            new_state = self._get_coap_value(self._state_cfg, payload)
            self._set_state(new_state, SRC_COAP)
        if extra_info_value_cfg:
            self.__update_info_values_coap(payload, extra_info_value_cfg)
        if self._info_value_cfg:
            self.__update_info_values_coap(payload, self._info_value_cfg)

    #Todo: remove
    def coap_get(self, payload, pos_list, default=None, channel=None):
        if pos_list is None:
            return default
        if not type(pos_list) is list:
            pos_list = [pos_list]
        for _pos in pos_list:
            ch = channel or self._channel or 0
            if _pos < 1000:
                pos = _pos + 10 * ch
            else:
                pos = _pos + 100 * ch
            if pos in payload:
                return payload[pos]
        return default

    def set_info_value(self, name, value, src):
        if value is None:
            return
        if self.info_values.get(name) != value:
            self.need_update = True
            self.info_values[name] = value
        self.info_values_updated[name] = datetime.now()
        if src == SRC_STATUS:
            self.info_values_status[name] = value
        elif src == SRC_COAP:
            self.info_values_coap[name] = value




"""Block is a physical device"""
# pylint: disable=broad-except, bare-except

from datetime import datetime
from .utils import shelly_http_get
from .switch import Switch
from .relay import Relay
from .powermeter import PowerMeter
from .sensor import (Sensor, BinarySensor, Flood, DoorWindow, ExtTemp,
                     ExtHumidity, Gas, TempSensor)
from .light import RGBW2W, RGBW2C, RGBWW, Bulb, Duo, Vintage
from .dimmer import Dimmer
from .roller import Roller
from .utils import exception_log
from .base import Base

from .const import (
    LOGGER,
    SENSOR_UNAVAILABLE_SEC,
    INFO_VALUE_DEVICE_TEMP,
    INFO_VALUE_CLOUD_STATUS,
    INFO_VALUE_CLOUD_ENABLED,
    INFO_VALUE_CLOUD_CONNECTED,
    INFO_VALUE_RSSI,
    INFO_VALUE_RSSI_LEVEL,
    INFO_VALUE_HAS_FIRMWARE_UPDATE,
    INFO_VALUE_FW_VERSION,
    INFO_VALUE_LATEST_FIRMWARE_VERSION,
    INFO_VALUE_LATEST_BETA_FW_VERSION,
    INFO_VALUE_PAYLOAD,
    INFO_VALUE_BATTERY,
    INFO_VALUE_ILLUMINANCE,
    INFO_VALUE_TILT,
    INFO_VALUE_VIBRATION,
    INFO_VALUE_TEMP,
    INFO_VALUE_GAS,
    INFO_VALUE_SENSOR,
    INFO_VALUE_TOTAL_WORK_TIME,
    ATTR_PATH,
    ATTR_FMT,
    ATTR_POS,
    ATTR_AUTO_SET,
    BLOCK_INFO_VALUES,
    SHELLY_TYPES,
    SRC_STATUS,
    RSSI_LEVELS
)

class Block(Base):
    def __init__(self, parent, block_id, block_type, ip_addr, discovery_src):
        super(Block, self).__init__()
        self.id = block_id
        self.unit_id = block_id
        self.type = block_type
        self.parent = parent
        self.ip_addr = ip_addr
        self.devices = []
        self.discovery_src = discovery_src
        self.protocols = []
        self.unavailable_after_sec = None
        #self.info_values = {}
        #self.info_values_updated = {}
        self.last_update_status_info = None
        self.reload = False
        self.last_updated = None #datetime.now()
        self.error = None
        self.discover_by_mdns = False
        self.discover_by_coap = False
        self.sleep_device = False
        self.payload = None
        self.settings = None
        self.exclude_info_values = []
        #self._info_value_cfg = None
        self._setup()
        self._available = None
        self.status_update_error_cnt = 0
        self.last_try_update_status = None
        self._last_friendly_name = None

    def update_coap(self, payload, ip_addr):
        self.ip_addr = ip_addr  # If changed ip
        self.last_updated = datetime.now()
        self._update_info_values_coap(payload, BLOCK_INFO_VALUES)   

        if self.payload:
            self.set_info_value(INFO_VALUE_PAYLOAD, self.payload, None)
     
        for dev in self.devices:
            dev.ip_addr = ip_addr
            dev._update_info_values_coap(payload)
            if hasattr(dev, 'update_coap'):
                dev.update_coap(payload)
            dev.raise_updated()
        if self.reload:
            self._reload_devices()
            for dev in self.devices:
                dev._update_info_values_coap(payload)
                if hasattr(dev, 'update_coap'):
                    dev.update_coap(payload)
                self.parent.add_device(dev, self.discovery_src)
            self.reload = False
        self.raise_updated()

    def loop(self):
        if self._info_value_cfg:
            for iv, cfg in self._info_value_cfg.items():
                if ATTR_AUTO_SET in cfg:
                    value = self.info_values.get(iv)
                    param = cfg.get(ATTR_AUTO_SET)
                    new_value = param[0]
                    if value != new_value:
                        time = self.info_values_updated.get(iv)
                        delay = param[1]
                        if time:
                            diff = datetime.now() - time
                            if diff.total_seconds() > delay:
                                self.set_info_value(iv, new_value, None)
                                self.raise_updated()

    def check_available(self):
        if self.available() != self._available:
            self._available = self.available()
            self.raise_updated(True)
            for dev in self.devices:
                dev.raise_updated(True)

    def update_status_information(self):
        """Update the status information."""
        self.last_update_status_info = datetime.now()

        if self.status_update_error_cnt >= 3:
            diff = (datetime.now()-self.last_try_update_status).total_seconds()
            if diff < 600 or (diff < 3600 and self.status_update_error_cnt >= 5):
                return

        self.last_try_update_status = datetime.now()

        LOGGER.debug("Get status from %s %s", self.id, self.friendly_name())
        success, status = self.http_get('/status', False)

        if not success or status == {}:
            self.status_update_error_cnt += 1
            return

        if 'poll' not in self.protocols:
            self.protocols.append("poll")

        self.status_update_error_cnt = 0
        self.last_updated = datetime.now()

        #Put status in info_values
        for name, cfg in BLOCK_INFO_VALUES.items():
            if name in self.exclude_info_values:
                continue
            self._update_info_value(name, status, cfg)

        if self._info_value_cfg:
            for name, cfg in self._info_value_cfg.items():
                self._update_info_value(name, status, cfg)

        cloud_status = 'disabled'
        if self.info_values.get(INFO_VALUE_CLOUD_ENABLED):
            if self.info_values.get(INFO_VALUE_CLOUD_CONNECTED):
                cloud_status = 'connected'
            else:
                cloud_status = 'disconnected'
        self.set_info_value(INFO_VALUE_CLOUD_STATUS, cloud_status, SRC_STATUS)

        rssi = self.info_values.get(INFO_VALUE_RSSI)
        rssi_level = self.info_values.get(INFO_VALUE_RSSI_LEVEL)
        for val, name in RSSI_LEVELS.items():
            if rssi >= val or ( rssi_level == name and rssi >= val - 2):
                rssi_level = name
                break
        self.set_info_value(INFO_VALUE_RSSI_LEVEL, rssi_level, SRC_STATUS)

        #self.info_values[INFO_VALUE_HAS_FIRMWARE_UPDATE] = self.has_fw_update()
        self.set_info_value(INFO_VALUE_LATEST_BETA_FW_VERSION,
                            self.latest_fw_version(True), SRC_STATUS)

        friendly_name = self.friendly_name()
        if self._last_friendly_name!=friendly_name:
            self._last_friendly_name!=friendly_name
            self.need_update

        force_update_devices = self.need_update #Block updated
        self.raise_updated()

        for dev in self.devices:
            try:
                if dev._state_cfg:
                    dev._update_state(status, dev._state_cfg)
                if dev._info_value_cfg:
                    for name, cfg in dev._info_value_cfg.items():
                        dev._update_info_value(name, status, cfg)
                dev.update_status_information(status)
                dev.raise_updated(force_update_devices)
            except Exception as ex:
                exception_log(ex, "Error update device status: {} {}", \
                    dev.id, dev.type)

    def http_get(self, url, log_error=True):
        """Send HTTP GET request"""
        success, res = shelly_http_get(self.ip_addr, url, \
                              self.parent.username, self.parent.password, \
                              log_error)
        return success, res

    def update_firmware(self, beta = False):
        """Start firmware update"""
        url = None
        if beta:
            url = self.parent._firmware_mgr.url(self.type, False)
        if url:
            self.http_get("/ota?url=" + url)
        else:
            self.http_get("/ota?update=1")

    def poll_settings(self):
        if self.type in SHELLY_TYPES and \
            SHELLY_TYPES[self.type].get('battery'):
                return
        success, settings = self.http_get("/settings")
        if success:
            self.settings = settings

    def _setup(self):
        #Get settings
        self.poll_settings()
        #Shelly BULB, ,Shelly Bulb RGBW GU10
        if self.type == 'SHBLB-1' or self.type == 'SHCL-255' or self.type == 'SHCB-1':
            self._add_device(Bulb(self))
        #Shelly 1
        elif self.type == 'SHSW-1' or self.type == 'SHSK-1':
            self._add_device(Relay(self, 0))
            self._add_device(Switch(self, 0))
            self._add_device(ExtTemp(self, 0), True)
            self._add_device(ExtTemp(self, 1), True)
            self._add_device(ExtTemp(self, 2), True)
            self._add_device(ExtHumidity(self, 0), True)
        #Shelly 1L
        elif self.type == 'SHSW-L':
            self._add_device(Relay(self, 0))
            self._add_device(Switch(self, 0))
        #Shelly 1 PM
        elif self.type == 'SHSW-PM':
            self._add_device(Relay(self, 0))
            self._add_device(PowerMeter(self, 0))
            self._add_device(Switch(self, 0))
            self._add_device(ExtTemp(self, 0), True)
            self._add_device(ExtTemp(self, 1), True)
            self._add_device(ExtTemp(self, 2), True)
            self._add_device(ExtHumidity(self, 0), True)
        #Shelly 2
        elif self.type == 'SHSW-21':
            if self.settings:
                if self.settings.get('mode') == 'roller':
                    self._add_device(Roller(self))
                else:
                    self._add_device(Relay(self, 1))
                    self._add_device(Relay(self, 2, consumption_channel = 0))
                self._add_device(Switch(self, 1))
                self._add_device(Switch(self, 2))
                self._add_device(PowerMeter(self, 0))
        #Shelly 2.5
        elif self.type == 'SHSW-25':
            if self.settings:
                if self.settings.get('mode') == 'roller':
                    self._add_device(Roller(self))
                    self._add_device(PowerMeter(self, 1))
                else:
                    self._add_device(Relay(self, 1))
                    self._add_device(Relay(self, 2))
                    self._add_device(PowerMeter(self, 1))
                    self._add_device(PowerMeter(self, 2))
                self._add_device(Switch(self, 1))
                self._add_device(Switch(self, 2))
                #self._add_device(InfoSensor(self, 'temperature'))
            #todo delayed reload
        #Shelly PLUG'S
        elif self.type == 'SHPLG-1' or self.type == 'SHPLG2-1' or \
              self.type == 'SHPLG-S' or self.type == 'SHPLG-U1':
            self._add_device(Relay(self, 0))
            self._add_device(PowerMeter(self, 0))
        elif self.type == 'SHEM':
            self._add_device(Relay(self, 0, include_power=False, em=True))
            self._add_device(PowerMeter(self, 1, voltage_to_block=True,
                                        em=True), major=True)
            self._add_device(PowerMeter(self, 2, voltage_to_block=True,
                                        em=True), major=True)
        #Shelly 3EM
        elif self.type == 'SHEM-3':
            self._add_device(Relay(self, 0, include_power=False, em=True))
            self._add_device(PowerMeter(self, 1, em=True), major=True)
            self._add_device(PowerMeter(self, 2, em=True), major=True)
            self._add_device(PowerMeter(self, 3, em=True), major=True)
        elif self.type == 'SH2LED-1':
            self._add_device(RGBW2W(self, 1))
            self._add_device(RGBW2W(self, 2))
        #Shelly 4 Pro
        elif self.type == 'SHSW-44':
            for channel in range(4):
                self._add_device(Relay(self, channel + 1))
                self._add_device(PowerMeter(self, channel + 1))
                self._add_device(Switch(self, channel + 1))
        elif self.type == 'SHRGBWW-01':
            self._add_device(RGBWW(self))
        #Shelly Dimmer
        elif self.type in ('SHDM-1', 'SHDM-2'):
            #self._info_value_cfg = {INFO_VALUE_DEVICE_TEMP : {ATTR_POS : 311}}
            self._add_device(Dimmer(self, [121, 1101], [111, 5101]))
            self._add_device(Switch(self, 1, position=[131, 2101]))
            self._add_device(Switch(self, 2, position=[131, 2101]))
            self._add_device(PowerMeter(self, 0, 4101, 4103))
        elif self.type == 'SHHT-1':
            self.sleep_device = True
            self.unavailable_after_sec = SENSOR_UNAVAILABLE_SEC
            self.exclude_info_values.append(INFO_VALUE_DEVICE_TEMP)
            self._add_device(TempSensor(self))
            self._add_device(Sensor(self, [44, 3103], 'humidity', 'hum/value'))
        #Shellyy RGBW2
        elif self.type == 'SHRGBW2':
            if self.settings:
                if self.settings.get('mode', 'color') == 'color':
                    self._add_device(RGBW2C(self))
                    self._add_device(PowerMeter(self, 0, position=[211, 4101]))
                else:
                    for channel in range(4):
                        self._add_device(RGBW2W(self, channel + 1))
                        self._add_device(PowerMeter(self, channel+1, [211, 4101]))
            self._add_device(Switch(self, 0))
            #todo else delayed reload
        #Shelly Flood
        elif self.type == 'SHWT-1':
            self.sleep_device = True
            self.unavailable_after_sec = SENSOR_UNAVAILABLE_SEC
            self._add_device(Flood(self))
            self._add_device(Sensor(self, 33, 'temperature', 'tmp/tC'))
        elif self.type == 'SHDW-1':
            self.sleep_device = True
            self.unavailable_after_sec = SENSOR_UNAVAILABLE_SEC
            self._info_value_cfg = {INFO_VALUE_BATTERY : {ATTR_POS : [77, 3111]},
                                    INFO_VALUE_TILT : {ATTR_POS : [88, 3109]},
                                    INFO_VALUE_VIBRATION : {ATTR_POS : [99, 6110],
                                                    ATTR_AUTO_SET: [0, 60]},
                                    INFO_VALUE_ILLUMINANCE: {ATTR_POS : [66, 3106]}
            }
            self._add_device(DoorWindow(self, [55, 3108]))
        elif self.type == 'SHDW-2':
            self.sleep_device = True
            self.exclude_info_values.append(INFO_VALUE_DEVICE_TEMP)
            self.unavailable_after_sec = SENSOR_UNAVAILABLE_SEC
            self._info_value_cfg = {#INFO_VALUE_BATTERY : {ATTR_POS : 3111},
                                    INFO_VALUE_TILT : {ATTR_POS : 3109, ATTR_PATH : 'accel/tilt'},
                                    INFO_VALUE_VIBRATION :
                                        {ATTR_POS : 6110,
                                         ATTR_PATH : 'accel/vibration',
                                         ATTR_AUTO_SET: [0, 60]},
                                    INFO_VALUE_TEMP: {ATTR_POS : 3101}, #Todo
                                    INFO_VALUE_ILLUMINANCE: {ATTR_POS : 3106, ATTR_PATH : 'lux/value'}
            }
            self._add_device(DoorWindow(self, 3108))
        elif self.type == 'SHBDUO-1':
            self._add_device(Duo(self))
            self._add_device(PowerMeter(self, 0, position=[141, 4101],
                                        tot_pos=[214, 4103]))
        elif self.type == 'SHVIN-1':
            self._add_device(Vintage(self))
            self._add_device(PowerMeter(self, 0, position=[141, 4101],
                                        tot_pos=[214, 4103]))
        elif self.type == 'SHBTN-1':
            self.sleep_device = True
            self.unavailable_after_sec = SENSOR_UNAVAILABLE_SEC
            self._add_device(Switch(self, 0, simulate_state=True,
                                    master_unit=True))
        elif self.type == 'SHIX3-1':
            self._add_device(Switch(self, 1, master_unit=True))
            self._add_device(Switch(self, 2))
            self._add_device(Switch(self, 3))
        elif self.type == 'SHGS-1':
            self._info_value_cfg = {INFO_VALUE_GAS : {ATTR_POS : 122},
                                    INFO_VALUE_SENSOR : {ATTR_POS : [118, 2101]}
            }
            self._add_device(Gas(self, 119))
        elif self.type == "SHUNI-1":
            self._add_device(Relay(self, 1, include_power=False))
            self._add_device(Relay(self, 2, include_power=False))
            self._add_device(Switch(self, 1))
            self._add_device(Switch(self, 2))            
            self._add_device(ExtTemp(self, 0), True)
            self._add_device(ExtTemp(self, 1), True)
            self._add_device(ExtTemp(self, 2), True)
            self._add_device(ExtHumidity(self, 0), True)
            self._add_device(Sensor(self, [3118], 'voltage', 'adcs/$/voltage'))
        elif self.type == 'SHAIR-1':
            self._info_value_cfg = {
                INFO_VALUE_TEMP: {ATTR_POS : 119,
                                  ATTR_PATH : 'ext_temperature/0/tC'},
                INFO_VALUE_TOTAL_WORK_TIME: {ATTR_POS : 121,
                                             ATTR_PATH : 'total_work_time'}
            }
            self._add_device(Relay(self, 0))
            self._add_device(PowerMeter(self, 0))
            self._add_device(Switch(self, 0))

    def _add_device(self, dev, lazy_load=False, major=False):
        dev.lazy_load = lazy_load
        dev.major_unit = major
        self.devices.append(dev)
        #self.parent.add_device(dev, self.discovery_src)
        return dev

    def _reload_devices(self):
        for device in self.devices:
            self.parent.remove_device(device, self.discovery_src)
            device.close()
        self.devices = []
        self._setup()

    def fw_version(self):
        return self.info_values.get(INFO_VALUE_FW_VERSION)

    def latest_fw_version(self, beta = False):
        if beta:
            return  self.parent._firmware_mgr.version(self.type, True)
        else:
            return self.info_values.get(INFO_VALUE_LATEST_FIRMWARE_VERSION)

    def has_fw_update(self, beta = False):
        latest = self.latest_fw_version(beta)
        current = self.fw_version()
        return latest and current and latest != current

    def friendly_name(self):
        try:
            if self.parent.cloud:
                name = self.parent.cloud.get_device_name(self.id.lower())
                if name:
                    return name
        except:
            pass
        return self.type_name() + ' - ' + self.id

    def room_name(self):
        if self.parent.cloud:
            return self.parent.cloud.get_room_name(self.id.lower())

    def type_name(self):
        """Type friendly name"""
        try:
            name = SHELLY_TYPES[self.type]['name']
        except:
            name = self.type
        return name

    def available(self):
        """Return if device available"""
        if self.unavailable_after_sec is None:
            return True
        if self.last_updated is None:
            return False
        diff = datetime.now() - self.last_updated
        return diff.total_seconds() <= self.unavailable_after_sec



import socket
import threading
import json

from .const import (
    LOGGER, SHELLY_TYPES
)

class MQTT_connection:
    def __init__(self, mqtt, connection, client_address):
        self._mqtt = mqtt
        self._connection = connection
        #connection.settimeout(5)
        self._client_address = client_address
        self._thread = threading.Thread(target=self._loop)
        self._thread.name = "MQTT connection"
        self._thread.daemon = True
        self._thread.start()

    def _loop(self):
        try:
            # Receive the data in small chunks and retransmit it
            while not self._mqtt._root.stopped.isSet():
                try:
                    head = self._connection.recv(1)
                    if not head:
                        break
                    pkg_type=head[0]>>4
                    flags=head[0]&0xF
                    length = 0
                    for s in range(0,4):
                        ldata = self._connection.recv(1)[0]
                        length += (ldata & 0x7F) << (s * 7)
                        if not ldata & 128:
                            break
                    LOGGER.debug("type=%d, flags=%d, length=%d", pkg_type, flags, length)

                    data = self._connection.recv(length) if length else None
                    if pkg_type==1:
                        msg = b'\x20\x04\x20\x00\x00\0x00'
                        self._connection.send(msg)
                    if pkg_type==3:
                        topic_len = (data[0]<<8) + data[1]
                        topic = data[2:2+topic_len].decode('ASCII') 
                        payload = data[2+topic_len:]
                        if topic=='shellies/announce':
                            payload = json.loads(payload)
                            ip_addr = payload['ip']
                            shelly_id = payload['id']
                            shelly_type, device_id = shelly_id.rsplit('-',1)
                            device_type = self._mqtt._mqtt_types.get(shelly_type)
                            if device_type:
                                self._mqtt._root.update_block(device_id, \
                                    device_type, ip_addr, 'MQTT-discovery', None)
                        else:
                            topics = topic.split('/')
                            shelly_id = topics[1]
                            shelly_type, device_id = shelly_id.rsplit('-',1)
                            device_type = self._mqtt._mqtt_types.get(shelly_type)
                            self._mqtt._root.update_block(device_id, \
                                    device_type, None, 'MQTT-data', None, True)
                    if pkg_type==12:
                        msg = b'\xD0\x00'
                        self._connection.send(msg)
                except socket.timeout:
                    pass
                except Exception as ex:
                    LOGGER.exception("Error receiving MQTT message")
                    break
        finally:
            #Clean up
            try:
                self._connection.close()
            except:
                pass
            try:
                self._mqtt._connections.remove(self)
            except:
                pass

class MQTT():

    def __init__(self, root):
        self._root = root
        self._thread = threading.Thread(target=self._loop)
        self._thread.name = "MQTT"
        self._thread.daemon = True
        self._socket = None
        self._connections = []
        self._mqtt_types = {}
        for key, item in SHELLY_TYPES.items():
            if 'mqtt' in item:
                self._mqtt_types[item['mqtt']]=key

    def start(self):
        if self._root.mqtt_port > 0:
            self._init_socket()
            self._thread.start()

    def _init_socket(self):
        # Create a TCP/IP socket
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.bind((self._root.bind_ip, self._root.mqtt_port))
        sock.listen(1)
        self._socket = sock

    def _loop(self):

        while not self._root.stopped.isSet():
            try:
                # Wait for a connection
                connection, client_address = self._socket.accept()
                conn = MQTT_connection(self, connection, client_address)
                self._connections.append(conn)
            except Exception as ex:
                LOGGER.exception("Error connect MQTT")

    def close(self):
        if self._socket:
            self._socket.close()



import threading
from datetime import timedelta, datetime
import time

class Loop(object):
    def __init__(self, name, root, interval, delay=5 ):
        self._loop_root = root
        self._loop_delay = delay
        self._loop_interval = interval
        self._loop_name = name
        self._loop_thread = None
        self._last_run = None

    def start_loop(self):
        self._loop_thread = threading.Thread(target=self._loop_start)
        self._loop_thread.name = self._loop_name
        self._loop_thread.daemon = True
        self._loop_thread.start()

    def _loop_start(self):
        try:
            self.loop_start()
        except:
            LOGGER.error("Error start loop %s, %s", self._loop_name, ex)
        while not self._loop_root.stopped.isSet():
            self.loop()

    def loop_start(self):
        pass

    def loop(self):
        if self._last_run is None or \
            datetime.now() - self._last_run \
                        > self._loop_interval:
            self._last_run = datetime.now()
            self.loop_timer()
        self._loop_root.stopped.wait(self._loop_delay)

    def loop_timer(self):
        pass


# -*- coding: utf-8 -*-

from .device import Device

from .const import (
    LOGGER,
    EFFECTS_BULB,
    EFFECTS_RGBW2,
    STATUS_RESPONSE_LIGHTS,
    STATUS_RESPONSE_LIGHTS_STATE,
    STATUS_RESPONSE_LIGHTS_BRIGHTNESS,
    STATUS_RESPONSE_LIGHTS_RED,
    STATUS_RESPONSE_LIGHTS_GREEN,
    STATUS_RESPONSE_LIGHTS_BLUE,
    STATUS_RESPONSE_LIGHTS_MODE,
    STATUS_RESPONSE_LIGHTS_POWER,
    INFO_VALUE_CURRENT_CONSUMPTION,
    INFO_VALUE_TOTAL_CONSUMPTION,
    #INFO_VALUE_SWITCH,
    ATTR_POS,
    ATTR_PATH,
    ATTR_FMT,
    SRC_COAP, SRC_STATUS
)

class Light(Device):
    def __init__(self, block):
        super(Light, self).__init__(block)

class LightWhite(Light):
    def __init__(self, block, channel, state_pos, bright_pos, 
                 temp_pos=None, power_pos=None):
        super(LightWhite, self).__init__(block)
        self.id = block.id
        self.state = None
        self.device_type = "LIGHT"
        self.url = "/light/0"
        if channel>0:
            self.id += "-" + str(channel)
            self._channel = channel-1
        else:
            self._channel = 0

        self.brightness = None
        self.color_temp = None

        self.state_pos = state_pos
        self.bright_pos = bright_pos
        self.temp_pos = temp_pos
        self.info_values = {}

        self.support_color_temp = False
        self._color_temp_min = None
        self._color_temp_max = None

        self._info_value_cfg = {
            # INFO_VALUE_SWITCH : { #Only one...
            #     ATTR_POS: [118, 2101],
            #     ATTR_PATH: 'inputs/$/input',
            #     ATTR_FMT: 'bool'
            # },
            INFO_VALUE_CURRENT_CONSUMPTION : {
                    ATTR_POS: power_pos or [141, 4101],
                    ATTR_PATH: 'meters/$/power',
                    ATTR_FMT: ['float']
            },
            INFO_VALUE_TOTAL_CONSUMPTION : {
                ATTR_POS: [214, 4103],
                ATTR_PATH: 'meters/$/total',
                ATTR_FMT: ['float','/60','round:2']
            }
        }

    def update_coap(self, payload):
        new_state = self.coap_get(payload, self.state_pos) == 1
        bright = self.coap_get(payload, self.bright_pos)
        if bright is not None:
            self.brightness = int(bright)
        color_temp = self.coap_get(payload, self.temp_pos)
        if color_temp is not None:
            self.color_temp = int(color_temp)

        values = {'brightness': self.brightness, "color_temp": self.color_temp}

        self._update(SRC_COAP, new_state, values)

    def update_status_information(self, status):
        """Update the status information."""
        new_state = None
        lights = status.get(STATUS_RESPONSE_LIGHTS)
        if lights:
            light = lights[self._channel]
            #self.brightness = int(light.get('gain', 0))
            self.brightness = \
                int(light.get(STATUS_RESPONSE_LIGHTS_BRIGHTNESS, 0))
            self.color_temp = int(light.get('temp', 0))
            new_state = light.get(STATUS_RESPONSE_LIGHTS_STATE, None)
            values = {'color_temp': self.color_temp, 'brightness': self.brightness}
            self._update(SRC_STATUS, new_state, values)

    def _send_data(self, state, brightness=None, color_temp=None):
        url = self.url + "?"

        if state is not None:
            if not state or brightness == 0:
                url += "turn=off"
                self._send_command(url)
                return
            url += "turn=on&"

        if brightness is not None:
            url += "brightness=" + str(brightness) + "&"

        if color_temp is not None:
            url += "temp=" + str(color_temp) + "&"

        self._send_command(url)

    def turn_on(self, brightness=None, color_temp=None):
        self._send_data(True, brightness, color_temp)

    def set_values(self, state=None, brightness=None, color_temp=None):
        self._send_data(state, brightness, color_temp)

    def turn_off(self):
        self._send_data(False)

    def get_dim_value(self):
        return self.brightness

    def set_dim_value(self, value):
        self._send_data(True, value)

    def get_color_temp_value(self):
        return self.color_temp

    def set_color_temp_value(self, value):
        self._send_data(True, color_temp=value)

class LightRGB(Light):
    def __init__(self, block, state_pos, channel=0, power_pos=None):
        super(LightRGB, self).__init__(block)
        self.id = block.id
        self.state = None
        self.device_type = "RGBLIGHT"
        self.url = "/light/0"

        self.mode = None
        self.brightness = None
        self.white_value = None
        self.rgb = None
        self.color_temp = None
        self.effect = None

        self.effects_list = None
        self.allow_switch_mode = True
        self.support_color_temp = False
        self.support_white_value = False

        self.state_pos = state_pos
        self.power_pos = power_pos
        if channel>0:
            self.id += "-" + str(channel)
            self._channel = channel-1
        else:
            self._channel = 0
        self.info_values = {}
        self._info_value_cfg = {
            # INFO_VALUE_SWITCH : {
            #     ATTR_POS: [118, 2101],
            #     ATTR_PATH: 'inputs/$/input',
            #     ATTR_FMT: 'bool'
            # },
            INFO_VALUE_CURRENT_CONSUMPTION : {
                    ATTR_POS: power_pos or [141, 4101],
                    ATTR_PATH: 'meters/$/power',
                    ATTR_FMT: ['float']
            },
            INFO_VALUE_TOTAL_CONSUMPTION : {
                ATTR_POS: [4103],
                ATTR_PATH: 'meters/$/total',
                ATTR_FMT: ['float','/60','round:2']
            }
        }

    def update_coap(self, payload):
        if not 9101 in payload:
            success, settings = self.block.http_get(self.url) #todo
            if not success:
                return
            self.mode = settings.get('mode', 'color')

            if self.mode == 'color':
                self.brightness = int(settings.get('gain', 0))
                self.white_value = int(settings.get('white', 0))
            else:
                self.brightness = int(settings.get('brightness', 0))

            self.color_temp = int(settings.get('temp', 0))
            self.effect = int(settings.get('effect', 0))

        else: #1.8.0
            self.mode = self.coap_get(payload, 9101)
            if self.mode == 'color':
                self.brightness = int(self.coap_get(payload, 5102))
            else:
                self.brightness = int(self.coap_get(payload, 5101))
            self.white_value = int(self.coap_get(payload, 5108))
            #Todo:self.color_temp = int(settings.get('temp', 0))

        #Todo
        # if mode != self.mode:
        #     if not self.allow_switch_mode and self.mode is not None:
        #         self._reload_block()
        #         return

        new_state = self.coap_get(payload, self.state_pos) == 1

        self.rgb = [self.coap_get(payload, [111, 5105]),
                    self.coap_get(payload, [121, 5106]),
                    self.coap_get(payload, [131, 5107])]

        # switch = self.coap_get(payload, [118, 2101])
        # if switch is not None:
        #     self.info_values['switch'] = switch > 0

        # power = self.coap_get(payload, [111, 5105])
        # if self.power_pos and self.power_pos in payload:
        #     self.info_values[INFO_VALUE_CURRENT_CONSUMPTION] \
        #         = self.coap_get(payload, self.power_pos)

        values = {'mode': self.mode, 'brightness': self.brightness,
                  'rgb': self.rgb, 'color_temp': self.color_temp,
                  'white_value': self.white_value,
                  'effect': self.effect}

        self._update(SRC_COAP, new_state, values)

    def update_status_information(self, status):
        """Update the status information."""
        new_state = None
        lights = status.get(STATUS_RESPONSE_LIGHTS)
        if lights:
            light = lights[self._channel]
            mode = light.get(STATUS_RESPONSE_LIGHTS_MODE, 'color')
            if mode != self.mode:
                if not self.allow_switch_mode and self.mode is not None:
                    self._reload_block()
                    return
            self.mode = mode
            if self.mode == 'color':
                self.brightness = int(light.get('gain', 0))
                self.white_value = int(light.get('white', 0))
                self.rgb = [int(light[STATUS_RESPONSE_LIGHTS_RED]),
                            int(light[STATUS_RESPONSE_LIGHTS_GREEN]),
                            int(light[STATUS_RESPONSE_LIGHTS_BLUE])]
            else:
                self.brightness = \
                    int(light.get(STATUS_RESPONSE_LIGHTS_BRIGHTNESS, 0))

            self.color_temp = int(light.get('temp', 0))

            self.effect = int(light.get('effect', 0))

            # if STATUS_RESPONSE_LIGHTS_POWER in light:
            #     self.info_values[INFO_VALUE_CURRENT_CONSUMPTION] \
            #         = light[STATUS_RESPONSE_LIGHTS_POWER]

            new_state = light.get(STATUS_RESPONSE_LIGHTS_STATE, None)
            values = {'mode': self.mode, 'brightness': self.brightness,
                      'rgb': self.rgb, 'color_temp': self.color_temp,
                      'white_value': self.white_value,
                      'effect': self.effect}
            self._update(SRC_STATUS, new_state, values)

    def _send_data(self, state, brightness=None, rgb=None, color_temp=None,
                   mode=None, effect=None, white_value=None):
        url = self.url + "?"

        if state is not None:
            if not state or brightness == 0:
                url += "turn=off"
                self._send_command(url)
                return
            url += "turn=on&"

        if mode is not None:
            self._send_command("/settings/?mode=" + mode)
        else:
            mode = self.mode

        if effect is not None:
            self._send_command("/color/0/?effect=" + str(effect))

        if brightness is not None:
            if mode == "white":
                url += "brightness=" + str(brightness) + "&"
            else:
                url += "gain=" + str(brightness) + "&"

        if white_value is not None:
            url += "white=" + str(white_value) + "&"

        if rgb is not None:
            url += "red=" + str(rgb[0]) + "&"
            url += "green=" + str(rgb[1]) + "&"
            url += "blue=" + str(rgb[2]) + "&"

        if color_temp is not None:
            url += "temp=" + str(color_temp) + "&"

        self._send_command(url)

    def turn_on(self, rgb=None, brightness=None, color_temp=None, mode=None,
                effect=None, white_value=None):
        self._send_data(True, brightness, rgb,
                        color_temp, mode, effect, white_value)

    def set_values(self, rgb=None, brightness=None, color_temp=None, mode=None,
                   effect=None, white_value=None):
        self._send_data(None, brightness, rgb,
                        color_temp, mode, effect, white_value)

    def turn_off(self):
        self._send_data(False)

    def get_dim_value(self):
        return self.brightness

    def set_dim_value(self, value):
        self._send_data(True, value)

    def get_white_value(self):
        return self.white_value

    def set_white_value(self, value):
        self._send_data(True, white_value=value)

class Bulb(LightRGB):
    def __init__(self, block):
        super(Bulb, self).__init__(block, [1101, 181])
        self.effects_list = EFFECTS_BULB
        self.support_color_temp = True

class RGBWW(LightRGB):
    def __init__(self, block):
        super(RGBWW, self).__init__(block, 151)
        self.support_color_temp = True

class RGBW2W(LightWhite):
    def __init__(self, block, channel):
        super(RGBW2W, self).__init__(block, channel, [161, 1101], [111, 5101],
                                     power_pos=[201, 4101])
        #self.id = self.id + '-' + str(channel)
        self.mode = "white"
        self.url = "/white/" + str(channel - 1)
        self.effects_list = None
        self.allow_switch_mode = False

    # def update_coap(self, payload):
    #     if 181 in payload:
    #         new_state = self.coap_get(payload, [151]) == 1
    #         self.brightness = self.coap_get(payload, [111])
    #         # if 118 in payload:
    #         #     self.info_values['switch'] = self.coap_get(payload, 118) > 0
    #         # if self.power_pos and self.power_pos in payload:
    #         #     self.info_values[INFO_VALUE_CURRENT_CONSUMPTION] \
    #         #         = self.coap_get(payload, self.power_pos)
    #         values = {'mode': self.mode, 'brightness': self.brightness}
    #                   #'rgb': self.rgb, 'temp': self.temp}
    #         self._update(new_state, values, None, self.info_values)
    #     else:
    #         self._reload_block()

class RGBW2C(LightRGB):
    def __init__(self, block):
        super(RGBW2C, self).__init__(block, [161, 1101], 0, [211, 4101])
        self.mode = "color"
        self.url = "/color/0"
        self.effects_list = EFFECTS_RGBW2
        self.allow_switch_mode = False
        self.support_white_value = True

class Duo(LightWhite):
    def __init__(self, block):
        super(Duo, self).__init__(block, 0, [121, 1101], [111, 5101], [131, 5103])
        self.support_color_temp = True
        self._color_temp_min = 2700
        self._color_temp_max = 6500
        self.is_sensor = True

class Vintage(LightWhite):
    def __init__(self, block):
        super(Vintage, self).__init__(block, 0, [121, 1101], [111, 5101])
        self.is_sensor = True



# -*- coding: utf-8 -*-
# pylint: disable=broad-except, bare-except

from .const import LOGGER, SHELLY_TYPES
from .base import Base

class Device(Base):
    def __init__(self, block):
        super(Device, self).__init__()
        self.block = block
        self.id = block.id
        self.unit_id = block.id
        self.type = block.type
        self.ip_addr = block.ip_addr
        self.is_device = True
        self.is_sensor = False
        self.sub_name = None
        self.state_values = None
        #self.sensor_values = None
        self.state = None
        self.device_type = None
        self.device_sub_type = None #Used to make sensors unique
        self.lazy_load = False
        self.device_nr = None
        self.master_unit = False
        self.head_unit = False
        self.ext_sensor = None

    def friendly_name(self):
        try:
            if self.block.parent.cloud:
                device_id = self.id.lower().split('-')
                name = None
                #add_nr = False
                idx = int(device_id[1]) if len(device_id) > 1 else 0
                name = self.block.parent.cloud.get_device_name(device_id[0],
                                                               idx,
                                                               self.ext_sensor)

                # if len(device_id) > 1 and int(device_id[1]) > 1:
                #     cloud_id = device_id[0] + '_' + str(int(device_id[1])-1)
                #     name = self.block.parent.cloud.get_device_name(cloud_id,
                #                                                self.ext_sensor)
                #     if not name:
                #         add_nr = True
                # if not name:
                #     name = \
                #       self.block.parent.cloud.get_device_name(device_id[0],
                #                                               self.ext_sensor)
                #     if add_nr:
                #         name += " - " + device_id[1]
                if name:
                    return name
        except Exception as ex:
            LOGGER.debug("Error look up name, %s", ex)
        name = self.type_name() + ' - ' + self.id
        #if self.device_nr:
        #   name += ' - ' + str(self.device_nr)
        return name

    def room_name(self):
        if self.block.parent.cloud:
            device_id = self.id.lower().split('-')
            room = None
            if len(device_id) > 1 and int(device_id[1]) > 1:
                room = self.block.parent.cloud.get_room_name(
                    device_id[0] + "_" + device_id[1])
            if room is None:
                room = self.block.parent.cloud.get_room_name(device_id[0])
            return room

    def type_name(self):
        """Friendly type name"""
        try:
            name = SHELLY_TYPES[self.type]['name']
        except:
            name = self.type
        if self.sub_name is not None:
            name = name + " (" + self.sub_name + ")"
        return name

    def _send_command(self, url):
        self.block.http_get(url)
        self.block.update_status_interval = None #Force update

    def available(self):
        return self.block.available()

    @property
    def protocols(self):
        return self.block.protocols

    def _update(self, src, new_state=None, new_state_values=None):
        LOGGER.debug("Update id:%s state:%s stateValue:%s", self.id, new_state, new_state_values)
        self._set_state(new_state, src)
        if new_state_values is not None:
            if self.state_values != new_state_values:
                self.state_values = new_state_values
                self.need_update = True
        #if new_values is not None:
        #    self.sensor_values = new_values
        #    self.need_update = True
        #if info_values is not None:
        #    self.info_values = info_values
        #    self.need_update = True
        if self.lazy_load:
            self.block.parent.callback_add_device(self)
        self.raise_updated()

    def update_status_information(self, _status):
        """Update the status information."""

    def fw_version(self):
        return self.block.fw_version()

    def close(self):
        self.cb_updated = []

    def _reload_block(self):
        self.block.reload = True

    def loop(self):
        pass




# -*- coding: utf-8 -*-

from .device import Device
from .utils import notNone
from threading import Timer
from datetime import datetime
from .const import (
    LOGGER,
    STATUS_RESPONSE_INPUTS,
    STATUS_RESPONSE_INPUTS_INPUT,
    STATUS_RESPONSE_INPUTS_EVENT,
    STATUS_RESPONSE_INPUTS_EVENT_CNT,
    SRC_COAP, SRC_STATUS
)

class Switch(Device):
    """Class to represent a power meter value"""
    def __init__(self, block, channel, 
                 position=None, simulate_state=False, master_unit=False):
        super(Switch, self).__init__(block)
        self.id = block.id
        self.master_unit = master_unit
        if channel > 0: #Todo: Move to base
            self.id += "-" + str(channel)
            self._channel = channel - 1
            self.device_nr = channel
        else:
            self._channel = 0
        self._position = notNone(position, [118, 2101])
        self._event_pos = [119, 2102]
        self._event_cnt_pos = [120, 2103]
        #if self._event_pos and type(self._event_pos) is not list:
        #    self._event_pos = [self._event_pos]
        #self._event_cnt_pos = event_cnt_pos
        self._simulate_state = simulate_state
        #self.sensor_values = {}
        self.device_type = "SWITCH"
        self.last_event = None
        self.event_cnt = None
        self.battery_bug_fix = False
        self.hold_delay = None #bug fix

    def update_coap(self, payload):
        """Get the power"""
        state = self.coap_get(payload, self._position)
        event_cnt = self.coap_get(payload, self._event_cnt_pos)
        self.battery_bug_fix = (self.coap_get(payload, [3112]) == 0 and event_cnt == 1)
        if self.battery_bug_fix:
            if self.hold_delay:
                diff = datetime.now() - self.hold_delay
                if diff.total_seconds() <= 10:
                    return
            self.hold_delay = datetime.now()
            event_cnt = (self.event_cnt or 0) + 1
            self.hold = True
        if not event_cnt is None and self.event_cnt != event_cnt:
            if self._simulate_state and self.event_cnt is not None:
                state = 1
                self.timer = Timer(1,self._turn_off)
                self.timer.start()
            self.last_event = self.coap_get(payload, self._event_pos)
            self.event_cnt = event_cnt
        if not state is None:
            state = bool(state)
        self._update(SRC_COAP, state, {'last_event' : self.last_event,
                                  'event_cnt' : self.event_cnt})

    def update_status_information(self, status):
        """Update the status information."""
        new_state = None
        if not self.battery_bug_fix:
            inputs = status.get(STATUS_RESPONSE_INPUTS)
            if inputs:
                value = inputs[self._channel]
                new_state = value.get(STATUS_RESPONSE_INPUTS_INPUT, None) != 0
                event_cnt = value.get(STATUS_RESPONSE_INPUTS_EVENT_CNT, None)
                if not event_cnt is None and self.event_cnt != event_cnt:
                    if self._simulate_state and self.event_cnt is not None:
                        new_state = True
                        self.timer = Timer(1,self._turn_off)
                        self.timer.start()
                    self.last_event = value.get(STATUS_RESPONSE_INPUTS_EVENT, None)
                    self.event_cnt = event_cnt
        self._update(SRC_STATUS, new_state, {'last_event' : self.last_event,
                                         'event_cnt' : self.event_cnt})

    def _turn_off(self):
        self._update(None, False)
        self.raise_updated()



# -*- coding: utf-8 -*-
# pylint: disable=broad-except, bare-except

import base64
import traceback
import json
import socket
from datetime import datetime
from .compat import s
try:
    import http.client as httplib
except:
    import httplib

from .const import (
    LOGGER
)

class timer():
    def __init__(self, interval):
        self._interval = interval
        self._last_time = None
    def check(self):
        if self._interval is not None:
            now = datetime.now()
            if self._last_time is None or \
                    now - self._last_time > self._interval:
                self._last_time = now
                return True
        return False

def exception_log(ex, _msg, *args):
    """Log exception"""
    msg = _msg.format(*args)
    try:
        msg += ", " + str(ex) + ", " + traceback.format_exc()
        LOGGER.exception(msg)
    except Exception as ex:
        LOGGER.error("Exception log error %s", ex)

def shelly_http_get(host, url, username, password, log_error=True):
    """Send HTTP GET request"""
    res = ""
    success = False
    conn = None
    try:
        LOGGER.debug("http://%s%s", host, url)
        conn = httplib.HTTPConnection(host, timeout=5)
        headers = {"Connection": "close"}
        if username is not None \
            and password is not None:
            combo = '%s:%s' % (username, password)
            auth = s(
                base64.b64encode(combo.encode()))  # .replace('\n', '')
            headers["Authorization"] = "Basic %s" % auth
        conn.request("GET", url, None, headers)
        resp = conn.getresponse()

        if resp.status == 200:
            body = resp.read()
            #LOGGER.debug("Body: %s", body)
            res = json.loads(s(body))
            success = True
            LOGGER.debug("http://%s%s - Ok", host, url)
        else:
            res = "Error, {} {} http://{}{}".format(
                resp.status, resp.reason, host, url)
            LOGGER.warning(res)
    except Exception as ex:
        success = False
        if (type(ex) == socket.timeout):
          msg = "Timeout connecting to http://" + host + url
          try:
            res = socket.gethostbyaddr(host)[0]
            msg += " [" + res + "]"
          except Exception as ex:
            pass
          LOGGER.error(msg)
        else:
          res = str(ex)
          if log_error:
              exception_log(ex, "Error http GET: http://{}{}", host, url)
          else:
              LOGGER.debug("Fail http GET: %s %s %s", host, url, ex)
    finally:
        if conn:
            conn.close()

    return success, res

def notNone(value, default):
    return default if value is None else value



# -*- coding: utf-8 -*-
# pylint: disable=broad-except, bare-except, invalid-name

import base64
from datetime import datetime, timedelta
import json
try:
    import asyncio
except:
    pass
import socket
import struct
import threading
import time

from .cloud import Cloud
from .block import Block
from .device import Device
from .utils import exception_log, timer
from .coap import CoAP
from .mqtt import MQTT
try:
    from .mdns import MDns
except:
    pass
from .firmware import Firmware_manager

#from .device.relay import Relay
#from .device.switch import Switch
#from .device.powermeter import Po

from .utils import shelly_http_get
from .compat import s, b, ba2c
from .const import (
    LOGGER,
    VERSION,
    STATUS_RESPONSE_RELAYS,
    STATUS_RESPONSE_RELAY_OVER_POWER,
    STATUS_RESPONSE_RELAY_STATE,
    STATUS_RESPONSE_METERS,
    STATUS_RESPONSE_METERS_POWER,
    SENSOR_UNAVAILABLE_SEC,
    INFO_VALUE_RSSI,
    INFO_VALUE_RSSI_LEVEL,
    INFO_VALUE_UPTIME,
    INFO_VALUE_OVER_POWER,
    INFO_VALUE_DEVICE_TEMP,
    INFO_VALUE_OVER_TEMPERATURE,
    INFO_VALUE_SSID,
    INFO_VALUE_HAS_FIRMWARE_UPDATE,
    INFO_VALUE_LATEST_FIRMWARE_VERSION,
    INFO_VALUE_LATEST_BETA_FW_VERSION,
    INFO_VALUE_FW_VERSION,
    INFO_VALUE_CLOUD_STATUS,
    INFO_VALUE_CLOUD_ENABLED,
    INFO_VALUE_CLOUD_CONNECTED,
    INFO_VALUE_MQTT_CONNECTED,
    INFO_VALUE_CURRENT_CONSUMPTION,
    INFO_VALUE_SWITCH,
    INFO_VALUE_BATTERY,
    ATTR_PATH,
    ATTR_FMT,
    BLOCK_INFO_VALUES,
    SHELLY_TYPES,
    EFFECTS_RGBW2,
    EFFECTS_BULB
)

__version__ = VERSION

class pyShelly():
    def __init__(self, loop=None):
        LOGGER.info("Init  %s", VERSION)
        self.stopped = threading.Event()
        self.blocks = {}
        self.devices = []
        self.cb_block_added = []
        self.cb_device_added = []
        self.cb_device_removed = []
        self.cb_save_cache = None
        self.cb_load_cache = None
        # Used if igmp packages not sent correctly
        self.igmp_fix_enabled = False
        self.mdns_enabled = False
        self.username = None
        self.password = None
        self.update_status_interval = None
        self._update_thread = None
        self._socket = None
        self.only_device_id = None
        self.tmpl_name = "{room} - {name}"

        self.cloud = None
        self.cloud_server = None
        self.cloud_auth_key = None

        self._coap = CoAP(self)
        self._mdns = None
        self._mqtt = MQTT(self)
        self._firmware_mgr =  Firmware_manager(self)
        self.host_ip = ''
        self.bind_ip = ''
        self.mqtt_port = 0
        self.firmware_url = None
        self.zeroconf = None

        self._shelly_by_ip = {}
        #self.loop = asyncio.get_event_loop()
        self.event_loop = None
        try:
            if loop:
                self.event_loop = loop
            else:
                self.event_loop = asyncio.get_event_loop()
        except:
            pass

        self._send_discovery_timer = timer(timedelta(seconds=60))
        self._check_by_ip_timer = timer(timedelta(seconds=60))

    def load_cache(self, name):
        if self.cb_load_cache:
            return self.cb_load_cache(name)
        return None

    def save_cache(self, name, data):
        if self.cb_save_cache:
            self.cb_save_cache(name, data)

    def set_cloud_settings(self, server, key, _firstTime=False):
        if self.cloud_auth_key==key and self.cloud_server==server and not _firstTime:
            return
        if self.cloud:
            self.cloud.stop()
            self.cloud=None
        if server and key:
            self.cloud = Cloud(self, server, key)
            self.cloud.start(not _firstTime)
        self.cloud_server = server
        self.cloud_auth_key = key

    def start(self):
        if self.mdns_enabled:
            self._mdns = MDns(self, self.zeroconf)
        self.set_cloud_settings(self.cloud_server, self.cloud_auth_key, True)
        if self._coap:
            self._coap.start()
        if self._mdns:
            self._mdns.start()
        if self._mqtt:
            self._mqtt.start()
        #asyncio.ensure_future(self._update_loop())
        self._update_thread = threading.Thread(target=self._update_loop)
        self._update_thread.name = "Update loop"
        self._update_thread.daemon = True
        self._update_thread.start()

    def version(self):
        return VERSION

    def close(self):
        self.stopped.set()
        if self._coap:
            self._coap.close()
        if self._mdns:
            self._mdns.close()
        if self._mqtt:
            self._mqtt.close()
        if self._update_thread is not None:
            self._update_thread.join()
        if self._socket:
            self._socket.close()

    def discover(self):
        if self._coap:
            self._coap.discover()

    def add_device_by_ip(self, ip_addr, src):
        if ip_addr not in self._shelly_by_ip:
            LOGGER.debug("Add device by host %s %s", ip_addr, src)
            self._shelly_by_ip[ip_addr] = {'done':False, 'src':src,
                                           'poll_block':None}
        else:
            block = self._shelly_by_ip[ip_addr]['poll_block']
            if block:
                if not src in block.protocols:
                    block.protocols.append(src)
                self._poll_block(block)

    def check_by_ip(self):
        for ip_addr in list(self._shelly_by_ip.keys()):
            data = self._shelly_by_ip[ip_addr]
            delay = data.get('delay')
            if delay:
                data['delay'] = delay - 1
                continue
            if not data['done']:
                done = False
                for id, block in self.blocks.items():
                    if block.ip_addr == ip_addr:
                        self._shelly_by_ip[ip_addr]['done'] = True
                        done = True
                if done:
                    continue
                success, settings = shelly_http_get(
                    ip_addr, "/settings", self.username, self.password, False)
                if success:
                    success, status = shelly_http_get(
                        ip_addr, "/status", self.username, self.password, False)
                    if success:
                        done = True
                        self._shelly_by_ip[ip_addr]['done'] = True
                        dev = settings["device"]
                        if not "hostname" in dev: #invalide device
                            LOGGER.info("Error adding device (missing hostname), %s %s",
                                ip_addr, data['src'])
                            continue
                        device_id = dev["hostname"].rpartition('-')[2]
                        device_type = dev["type"]
                        ip_addr = status["wifi_sta"]["ip"]
                        LOGGER.debug("Add device from IP, %s, %s, %s", device_id, device_type, ip_addr)
                        self.update_block(device_id, device_type, ip_addr,
                                          data['src'],
                                          None)
                        if device_id in self.blocks:
                            block = self.blocks[device_id]
                            if block and block.sleep_device:
                                self._shelly_by_ip[ip_addr]['poll_block'] \
                                    = block
                        self._shelly_by_ip[ip_addr]['errors'] = 0
                if not done:
                    errors = data.get('errors', 0)
                    self._shelly_by_ip[ip_addr]['errors'] = errors + 1
                    self._shelly_by_ip[ip_addr]['delay'] = 60
                    if errors > 5:
                        self._shelly_by_ip[ip_addr]['delay'] = 600
                    LOGGER.info("Error adding device, %s %s",
                                ip_addr, data['src'])

    def add_device(self, dev, discovery_src):
        LOGGER.debug('Add device')
        dev.discovery_src = discovery_src
        self.devices.append(dev)
        if not dev.lazy_load:
            self.callback_add_device(dev)

    def callback_add_device(self, dev):
        if hasattr(dev, 'discovery_src'):
            for callback in self.cb_device_added:
                dev.lazy_load = False
                callback(dev, dev.discovery_src)

    def remove_device(self, dev, discovery_src):
        LOGGER.debug('Remove device')
        self.devices.remove(dev)
        for callback in self.cb_device_removed:
            callback(dev, discovery_src)

    def update_block(self, block_id, device_type, ipaddr, src, payload,
                     force_poll=False):
        if self.only_device_id is not None and \
            block_id != self.only_device_id:
            return

        block_added = False
        if block_id not in self.blocks:
            if not ipaddr:
                return
            block = self.blocks[block_id] = \
                Block(self, block_id, device_type, ipaddr, src)
            block_added = True

        block = self.blocks[block_id]

        if not src in block.protocols:
            block.protocols.append(src)

        if payload:
            data = {d[1]:d[2] for d in json.loads(payload)['G']}
            block.payload = payload
            block.update_coap(data, ipaddr)

        if block_added:
            for callback in self.cb_block_added:
                callback(block)
            for device in block.devices:
                self.add_device(device, block.discovery_src)

        if block.sleep_device or force_poll:
            self._poll_block(block, force_poll)

    def _update_loop(self):
        LOGGER.debug("Start update loop, %s sec", self.update_status_interval)
        while not self.stopped.isSet():
            try:
                #LOGGER.debug("Checking blocks")
                if self._check_by_ip_timer.check():
                    self.check_by_ip()
                if self._send_discovery_timer.check():
                    self.discover()

                for key in list(self.blocks.keys()):
                    block = self.blocks[key]

                    if not block.sleep_device:
                        self._poll_block(block)

                    block.check_available()
                    block.loop()

                self.stopped.wait(5)
            except Exception as ex:
                exception_log(ex, "Error update loop")

    def _poll_block(self, block, force=False):
        now = datetime.now()
        if force or \
            (self.update_status_interval is not None and \
            (block.last_update_status_info is None or \
            now - block.last_update_status_info \
                > self.update_status_interval)):
            LOGGER.debug("Polling block, %s %s", block.id, block.type)
            block.last_update_status_info = now
            t = threading.Thread(
                target=block.update_status_information)
            t.name = "Poll status"
            t.daemon = True
            t.start()



# -*- coding: utf-8 -*-

from .device import Device

from .const import (
    LOGGER,
    STATUS_RESPONSE_LIGHTS,
    STATUS_RESPONSE_LIGHTS_STATE,
    STATUS_RESPONSE_LIGHTS_BRIGHTNESS,
    STATUS_RESPONSE_INPUTS,
    STATUS_RESPONSE_INPUTS_INPUT,
    INFO_VALUE_SWITCH,
    STATUS_RESPONSE_METERS,
    STATUS_RESPONSE_METERS_POWER,
    INFO_VALUE_CURRENT_CONSUMPTION,
    SRC_COAP, SRC_STATUS
)

class Dimmer(Device):
    def __init__(self, block, state_pos, dim_pos):
        super(Dimmer, self).__init__(block)
        self.id = block.id
        self.device_type = "DIMMER"
        self.url = "/light/0"
        self.state = None
        self.brightness = None
        self.state_pos = state_pos
        self.dim_pos = dim_pos
        self.info_values = {}
        self.is_sensor = True

    def update_coap(self, payload):
        new_state = self.coap_get(payload, self.state_pos) == 1
        self.brightness = self.coap_get(payload, self.dim_pos)
        values = {'brightness': self.brightness}
        for idx in range(0, 2):
            value = self.coap_get(payload, [131, 2101], channel=idx)
            if value is not None:
                self.set_info_value(INFO_VALUE_SWITCH + "_" + str(idx+1),
                                    value > 0, SRC_COAP)
        consumption = self.coap_get(payload, [4101])
        if consumption is not None:
            self.set_info_value(INFO_VALUE_CURRENT_CONSUMPTION,
                                    float(consumption), SRC_COAP)

            #if value is not None:
            #    self.info_values[INFO_VALUE_SWITCH + "_" + str(idx+1)] = value > 0
        #todo, read consumption when firmware fixed
        self._update(SRC_COAP,new_state, values)

    def update_status_information(self, status):
        """Update the status information."""
        new_state = None
        lights = status.get(STATUS_RESPONSE_LIGHTS)
        values = {}
        if lights:
            light = lights[0]
            new_state = light.get(STATUS_RESPONSE_LIGHTS_STATE, None)
            self.brightness = \
                light.get(STATUS_RESPONSE_LIGHTS_BRIGHTNESS, None)
            values['brightness'] = self.brightness
        inputs = status.get(STATUS_RESPONSE_INPUTS)
        if inputs:
            for idx in range(0, 2):
                input = inputs[idx]
                value = input.get(STATUS_RESPONSE_INPUTS_INPUT)
                if value is not None:
                    self.set_info_value(INFO_VALUE_SWITCH + "_" + str(idx+1),
                                         value > 0, SRC_STATUS)
        meters = status.get(STATUS_RESPONSE_METERS)
        if meters and len(meters) > 0:
            meter = meters[0]
            value = meter.get(STATUS_RESPONSE_METERS_POWER)
            if value is not None:
                self.set_info_value(INFO_VALUE_CURRENT_CONSUMPTION,
                                     float(value), SRC_STATUS)

        self._update(SRC_STATUS, new_state, values)

    def _send_data(self, state, brightness=None):
        url = self.url + "?"
        if state is not None:
            if not state or brightness == 0:
                url += "turn=off"
                self._send_command(url)
                return
            url += "turn=on&"
        if brightness is not None:
            url += "brightness=" + str(brightness) + "&"

        self._send_command(url)

    def turn_on(self, brightness=None):
        self._send_data(True, brightness)

    def turn_off(self):
        self._send_data(False)

    def get_dim_value(self):
        return self.brightness

    def set_dim_value(self, value):
        self._send_data(True, value)



# -*- coding: utf-8 -*-
"""123"""

from .device import Device

from .utils import notNone

from .const import (
    INFO_VALUE_CURRENT_CONSUMPTION,
    INFO_VALUE_TOTAL_CONSUMPTION,
    INFO_VALUE_OVER_POWER,
    INFO_VALUE_SWITCH,
    STATUS_RESPONSE_RELAYS,
    STATUS_RESPONSE_RELAY_OVER_POWER,
    STATUS_RESPONSE_RELAY_STATE,
    STATUS_RESPONSE_METERS,
    STATUS_RESPONSE_METERS_POWER,
    STATUS_RESPONSE_METERS_TOTAL,
    STATUS_RESPONSE_INPUTS,
    STATUS_RESPONSE_INPUTS_INPUT,
    ATTR_POS,
    ATTR_FMT,
    ATTR_PATH,
    ATTR_CHANNEL
)

class Relay(Device):
    def __init__(self, block, channel, consumption_channel = None,
                 include_power=True, em=False):
        #Todo: common_consumption
        super(Relay, self).__init__(block)
        self.id = block.id
        if channel > 0:
            self.id += '-' + str(channel)
            self._channel = ch = channel - 1
            self.device_nr = channel
        else:
            self._channel = ch = 0
        self.state = None
        self.device_type = "RELAY"
        self.is_sensor = include_power
        self._state_cfg = {
            ATTR_POS: [112, 1101],
            ATTR_PATH: 'relays/$/ison',
            ATTR_FMT: 'bool'
        }
        self._info_value_cfg = {
            #Todo add total used and returned 4106, 4107
            INFO_VALUE_SWITCH : {
                ATTR_POS: [118, 2101],
                ATTR_PATH: 'inputs/$/input',
                ATTR_FMT: 'bool'
            },
            INFO_VALUE_OVER_POWER : {
                ATTR_POS: [6102],
                ATTR_PATH: 'relays/$/overpower',
                ATTR_FMT: 'bool'
            }
        }
        if include_power:
            meters = "emeters" if em else "meters"
            divider = None if em else '/60'
            self._info_value_cfg.update({
                INFO_VALUE_CURRENT_CONSUMPTION : {
                    ATTR_POS: [111, 4101, 4102, 4105],
                    ATTR_CHANNEL: notNone(consumption_channel, self._channel),
                    ATTR_PATH: meters + '/$/power',
                    ATTR_FMT: ['float']
                },
                INFO_VALUE_TOTAL_CONSUMPTION : {
                    ATTR_POS: [4103, 4104, 4106],
                    ATTR_CHANNEL: notNone(consumption_channel, self._channel),
                    ATTR_PATH: meters + '/$/total',
                    ATTR_FMT: ['float', divider,'round:2']
                }
            })
        # else:
        #     _info_value_cfg.update({
        #         INFO_VALUE_CURRENT_CONSUMPTION : {
        #             ATTR_POS: [111, 4105],
        #             ATTR_CHANNEL: self._channel,
        #             ATTR_PATH: 'meters/$/power',
        #             ATTR_FMT: ['float']
        #         },
        #         INFO_VALUE_TOTAL_CONSUMPTION : {
        #             ATTR_POS: [4106],
        #             ATTR_CHANNEL: notNone(consumption_channel, self._channel),
        #             ATTR_PATH: 'emeters/$/total',
        #             ATTR_FMT: ['float']
        #         }
        #     )

    def as_light(self):
        if self.block.parent.cloud:
            usage = self.block.parent.cloud.get_relay_usage(self.unit_id,
                                                           self._channel)
            return usage == 'light'

    # def update_coap(self, payload):
    #     new_state = self.coap_get(payload, self._pos) == 1
    #     #if self._power_idx is not None:
    #     #    consumption = data.get(self._power_idx)
    #     #    self.info_values[INFO_VALUE_CURRENT_CONSUMPTION] = round(consumption)
    #     #if self._switch_idx is not None:
    #     #    switch_state = data.get(self._switch_idx)
    #     #    self.info_values[INFO_VALUE_SWITCH] = switch_state > 0
    #     self._update(new_state, None, None, self.info_values)

    # def update_status_information(self, status):
    #     """Update the status information."""
    #     new_state = None
    #     relays = status.get(STATUS_RESPONSE_RELAYS)
    #     if relays:
    #         relay = relays[self._channel]
    #         # if relay.get(STATUS_RESPONSE_RELAY_OVER_POWER) is not None:
    #         #     self.info_values[INFO_VALUE_OVER_POWER] = \
    #         #         relay.get(STATUS_RESPONSE_RELAY_OVER_POWER)
    #         if relay.get(STATUS_RESPONSE_RELAY_STATE) is not None:
    #             new_state = relay.get(STATUS_RESPONSE_RELAY_STATE)
    #     # if self._power_idx:
    #     #     meters = status.get(STATUS_RESPONSE_METERS)
    #     #     if meters and len(meters) > 0:
    #     #         idx = min(self._channel, len(meters)-1)
    #     #         meter = meters[idx]
    #     #         if meter.get(STATUS_RESPONSE_METERS_POWER) is not None:
    #     #             self.info_values[INFO_VALUE_CURRENT_CONSUMPTION] = \
    #     #                 round(float(meter.get(STATUS_RESPONSE_METERS_POWER)))
    #     #         if meter.get(STATUS_RESPONSE_METERS_TOTAL) is not None:
    #     #             self.info_values[INFO_VALUE_TOTAL_CONSUMPTION] = \
    #     #             round(float(meter.get(STATUS_RESPONSE_METERS_TOTAL)) / 60)

    #     # inputs = status.get(STATUS_RESPONSE_INPUTS)
    #     # if inputs:
    #     #     value = inputs[self._channel]
    #     #     if value.get(STATUS_RESPONSE_INPUTS_INPUT) is not None:
    #     #         self.info_values[INFO_VALUE_SWITCH] = \
    #     #             value.get(STATUS_RESPONSE_INPUTS_INPUT) > 0
    #     self._update(new_state, info_values=self.info_values)

    def turn_on(self):
        self._send_command("/relay/" + str(self._channel) + "?turn=on")

    def turn_off(self):
        self._send_command("/relay/" + str(self._channel) + "?turn=off")



# -*- coding: utf-8 -*-

from .device import Device
from .const import (
    INFO_VALUE_VIBRATION,
    INFO_VALUE_HUMIDITY,
    ATTR_PATH,
    ATTR_FMT,
    ATTR_POS
)

class Sensor(Device):
    def __init__(self, block, pos, device_type, path, index=None):
        super(Sensor, self).__init__(block)
        self.id = block.id
        self._channel = index or 0
        if index is not None:
            self.id = self.id + "-" + str(index + 1)
            self.device_nr = index + 1
        self.state = None
        self.device_type = "SENSOR"
        self.device_sub_type = device_type
        self.is_sensor = True
        self.is_device = False
        #self._pos = pos
        self.sensor_type = device_type
        #elf._status_attr = path
        self._state_cfg = {
            ATTR_POS: pos,
            ATTR_PATH: path,
            ATTR_FMT: self.format
        }

    def format(self, value):
        return float(value)

    # def update_coap(self, payload):
    #     if self._pos:
    #         value = self.coap_get(payload, self._pos)
    #         if value is not None:
    #             self._update(self.format(value))

    # def update_status_information(self, status):
    #     """Update the status information."""
    #     data = status
    #     for key in self._status_attr.split('/'):
    #         data = data.get(key) if data is not None else None
    #     if data is not None:
    #         #self._update(None, None, {self.sensor_type:self.format(data)})
    #         self._update(self.format(data))

class TempSensor(Sensor):
    """Class to represent a external temp sensor"""
    def __init__(self, block):
        super(TempSensor, self).__init__(block, [33, 3101], \
            'temperature', 'tmp/tC')
        self._info_value_cfg = { 
            INFO_VALUE_HUMIDITY : { #Used in tellstick
                ATTR_POS: [44, 3103],
                ATTR_PATH: 'hum/value',
                ATTR_FMT: "float"
            }
        }

class ExtTemp(Sensor):
    """Class to represent a external temp sensor"""
    def __init__(self, block, idx):
        super(ExtTemp, self).__init__(block, [119, 3101], \
            'temperature', 'ext_temperature/' + str(idx) + "/tC", idx)
        self._info_value_cfg = { 
            INFO_VALUE_HUMIDITY : { #Used in tellstick
                ATTR_POS: [120, 3103],
                ATTR_PATH: 'ext_humidity/' + str(idx) + "/hum",
                ATTR_FMT: "float"
            }
        }
        self.sleep_device = False
        self.ext_sensor = idx

class ExtHumidity(Sensor):
    """Class to represent a external humidity sensor"""
    def __init__(self, block, idx):
        super(ExtHumidity, self).__init__(block, [120, 3103], \
            'humidity', 'ext_humidity/' + str(idx) + "/hum", idx)
        self.sleep_device = False
        self.ext_sensor = 1

class BinarySensor(Sensor):
    """Abstract class to represent binary sensor"""
    def __init__(self, block, pos, device_type, status_attr):
        super(BinarySensor, self).__init__(block, pos, device_type, status_attr)
        self.device_type = "BINARY_SENSOR"

    def format(self, value):
        if value == 'open' or value == 'mild' or value == 'heavy' :
            return True
        if value == 'close' or value == 'none':
            return False
        return bool(value)

class Flood(BinarySensor):
    """Class to represent a flood sensor"""
    def __init__(self, block):
        super(Flood, self).__init__(block, 23, 'flood', 'flood')
        self.sleep_device = True

class DoorWindow(BinarySensor):
    """Class to represent a door/window sensor"""
    def __init__(self, block, position):
        super(DoorWindow, self).__init__(
            block, position, 'door_window', 'sensor/state')
        self.sleep_device = True

class Gas(BinarySensor):
    """Class to represent a Gas sensor"""
    def __init__(self, block, position):
        super(Gas, self).__init__(
            block, position, 'gas', 'gas_sensor/alarm_state')




# -*- coding: utf-8 -*-

from .device import Device

from .const import (
    LOGGER,
    INFO_VALUE_CURRENT_CONSUMPTION,
    INFO_VALUE_TOTAL_CONSUMPTION,
    STATUS_RESPONSE_ROLLERS,
    STATUS_RESPONSE_ROLLERS_STATE,
    STATUS_RESPONSE_ROLLERS_LAST_DIR,
    STATUS_RESPONSE_ROLLERS_POSITION,
    STATUS_RESPONSE_ROLLERS_POWER,
    STATUS_RESPONSE_ROLLERS_TOTAL,
    ATTR_POS,
    ATTR_FMT,
    ATTR_PATH,
    SRC_COAP, SRC_STATUS
)

class Roller(Device):
    def __init__(self, block):
        super(Roller, self).__init__(block)
        self.id = block.id
        self.device_type = "ROLLER"
        self.state = None
        self.position = None
        self.is_sensor = True
        self.sub_name = "Roller"
        self.support_position = False
        self.motion_state = ""
        self.last_direction = ""
        self.info_values = {}
        #success, settings = self.block.http_get("/roller/0") #Todo move
        #if success:
        #    self.support_position = settings.get("positioning", False)
        self._info_value_cfg = {
            INFO_VALUE_TOTAL_CONSUMPTION : {
                ATTR_POS: [4104],
                ATTR_PATH: 'meters/$/total',
                ATTR_FMT: ['float','/60','round:2']
            }
        }

    def _set_pos(self, pos):
        if pos is None or pos < 0 or pos > 100:
            self.position = None
            self.support_position = False
        else:
            self.position = pos
            self.support_position = True

    def update_coap(self, payload):
        """Update current state"""
        self.motion_state = self.coap_get(payload, [1102]) or "stop"
        if self.coap_get(payload, [112]):
            self.motion_state = "open"
        if self.coap_get(payload, [122]):
            self.motion_state = "close"
        if self.motion_state and self.motion_state!="stop":
            self.last_direction = self.motion_state
        self._set_pos(self.coap_get(payload, [113, 1103]))
        self.set_info_value(INFO_VALUE_CURRENT_CONSUMPTION,
            self.coap_get(payload, 111, 0) + self.coap_get(payload, 121, 0) + \
                self.coap_get(payload, 4102, 0), SRC_COAP)
        state = self.position != 0
        self._update(SRC_COAP, state)

    def update_status_information(self, status):
        """Update the status information."""
        rollers = status.get(STATUS_RESPONSE_ROLLERS)
        if rollers:
            roller = rollers[0]
            self.support_position = roller.get("positioning", False)
            self.motion_state = roller[STATUS_RESPONSE_ROLLERS_STATE]
            self.last_direction = roller[STATUS_RESPONSE_ROLLERS_LAST_DIR]
            self._set_pos(roller[STATUS_RESPONSE_ROLLERS_POSITION])
            self.set_info_value(INFO_VALUE_CURRENT_CONSUMPTION,
                                roller[STATUS_RESPONSE_ROLLERS_POWER],
                                SRC_STATUS)
            state = self.position != 0
            self._update(SRC_STATUS, state)

    def up(self):
        self._send_command("/roller/0?go=open")

    def down(self):
        self._send_command("/roller/0?go=close")

    def stop(self):
        self._send_command("/roller/0?go=stop")

    def set_position(self, pos):
        if self.support_position:
            self.position=pos
            self.raise_updated(True)
            self._send_command("/roller/0?go=to_pos&roller_pos=" + str(pos))



# -*- coding: utf-8 -*-

import logging

LOGGER = logging.getLogger('pyShelly')

NAME = "pyShelly"
VERSION = "0.2.15"

COAP_IP = "224.0.1.187"
COAP_PORT = 5683

MDNS_IP = "224.0.0.251"
MDNS_PORT = 5353

REGEX_VER = r"^20(\d{6}).+v(\d+\.\d+\.\d+(-rc\d)?)(_.*)?@"

"""Define constants for result from /status response from device"""
STATUS_RESPONSE_RELAYS = 'relays'
STATUS_RESPONSE_RELAY_OVER_POWER = 'overpower'
STATUS_RESPONSE_RELAY_STATE = 'ison'

STATUS_RESPONSE_METERS = 'meters'
STATUS_RESPONSE_METERS_POWER = 'power'
STATUS_RESPONSE_METERS_TOTAL = 'total'
STATUS_RESPONSE_METERS_VOLTAGE = 'voltage'
STATUS_RESPONSE_METERS_PF = 'pf'
STATUS_RESPONSE_METERS_CURRENT = 'current'
STATUS_RESPONSE_METERS_TOTAL_RETURNED = 'total_returned'

STATUS_RESPONSE_EMETERS = 'emeters'

STATUS_RESPONSE_INPUTS = 'inputs'
STATUS_RESPONSE_INPUTS_INPUT = 'input'
STATUS_RESPONSE_INPUTS_EVENT = 'event'
STATUS_RESPONSE_INPUTS_EVENT_CNT = 'event_cnt'

STATUS_RESPONSE_LIGHTS = 'lights'
STATUS_RESPONSE_LIGHTS_STATE = 'ison'
STATUS_RESPONSE_LIGHTS_BRIGHTNESS = 'brightness'
STATUS_RESPONSE_LIGHTS_WHITE = 'white'
STATUS_RESPONSE_LIGHTS_MODE = 'mode'
STATUS_RESPONSE_LIGHTS_RED = 'red'
STATUS_RESPONSE_LIGHTS_GREEN = 'green'
STATUS_RESPONSE_LIGHTS_BLUE = 'blue'
STATUS_RESPONSE_LIGHTS_POWER = 'power'

STATUS_RESPONSE_ROLLERS = 'rollers'
STATUS_RESPONSE_ROLLERS_STATE = 'state'
STATUS_RESPONSE_ROLLERS_LAST_DIR = 'last_direction'
STATUS_RESPONSE_ROLLERS_POSITION = 'current_pos'
STATUS_RESPONSE_ROLLERS_POWER = 'power'
STATUS_RESPONSE_ROLLERS_TOTAL = 'total'

SENSOR_UNAVAILABLE_SEC = 3600 * 13 #13 hours

INFO_VALUE_RSSI = 'rssi'
INFO_VALUE_RSSI_LEVEL = 'rssi_level'
INFO_VALUE_UPTIME = 'uptime'
INFO_VALUE_OVER_POWER = 'over_power'
INFO_VALUE_DEVICE_TEMP = 'device_temp'
INFO_VALUE_OVER_TEMPERATURE = 'over_temp'
INFO_VALUE_SSID = 'ssid'
INFO_VALUE_HAS_FIRMWARE_UPDATE = 'has_firmware_update'
INFO_VALUE_LATEST_FIRMWARE_VERSION = 'latest_fw_version'
INFO_VALUE_LATEST_BETA_FW_VERSION = 'latest_beta_fw_version'
INFO_VALUE_FW_VERSION = 'firmware_version'
INFO_VALUE_CLOUD_STATUS = 'cloud_status'
INFO_VALUE_CLOUD_ENABLED = 'cloud_enabled'
INFO_VALUE_CLOUD_CONNECTED = 'cloud_connected'
INFO_VALUE_MQTT_CONNECTED = 'mqtt_connected'
INFO_VALUE_CURRENT_CONSUMPTION = 'current_consumption'
INFO_VALUE_SWITCH = 'switch'
INFO_VALUE_BATTERY = 'battery'
INFO_VALUE_PAYLOAD = 'payload'
INFO_VALUE_TOTAL_CONSUMPTION = 'total_consumption'
INFO_VALUE_TOTAL_RETURNED = 'total_returned'
INFO_VALUE_VOLTAGE = 'voltage'
INFO_VALUE_POWER_FACTOR = 'power_factor'
INFO_VALUE_CURRENT = 'current'
INFO_VALUE_TILT = 'tilt'
INFO_VALUE_VIBRATION = 'vibration'
INFO_VALUE_TEMP = 'temperature'
INFO_VALUE_HUMIDITY = 'humidity'
INFO_VALUE_ILLUMINANCE = 'illuminance'
INFO_VALUE_GAS = 'gas'
INFO_VALUE_SENSOR = 'sensor'
INFO_VALUE_TOTAL_WORK_TIME = 'total_work_time'

ATTR_PATH = 'path'
ATTR_FMT = 'fmt'
ATTR_POS = 'pos'
ATTR_CHANNEL = 'channel'
ATTR_AUTO_SET = 'auto_set'

SRC_COAP = 1
SRC_STATUS = 2

BLOCK_INFO_VALUES = {
    INFO_VALUE_SSID : {ATTR_PATH :'wifi_sta/ssid'},
    INFO_VALUE_RSSI : {ATTR_PATH : 'wifi_sta/rssi'},
    INFO_VALUE_UPTIME : {ATTR_PATH : 'uptime'},
    INFO_VALUE_DEVICE_TEMP : {ATTR_PATH : 'tmp/tC', ATTR_POS: [311, 3104]}, #, ATTR_FMT : 'round'},
    INFO_VALUE_OVER_TEMPERATURE : {ATTR_PATH : 'overtemperature', ATTR_POS: 6101},
    INFO_VALUE_HAS_FIRMWARE_UPDATE : {ATTR_PATH : 'update/has_update'},
    INFO_VALUE_LATEST_FIRMWARE_VERSION : {ATTR_PATH : 'update/new_version',
                                          ATTR_FMT : 'ver'},
    INFO_VALUE_FW_VERSION : {ATTR_PATH : 'update/old_version',
                             ATTR_FMT : 'ver'},
    INFO_VALUE_CLOUD_ENABLED : {ATTR_PATH : 'cloud/enabled'},
    INFO_VALUE_CLOUD_CONNECTED : {ATTR_PATH : 'cloud/connected'},
    INFO_VALUE_MQTT_CONNECTED : {ATTR_PATH : 'mqtt/connected'},
    #INFO_VALUE_CURRENT_CONSUMPTION : {ATTR_PATH : 'consumption'},
    INFO_VALUE_VOLTAGE : {ATTR_PATH : 'voltage', ATTR_FMT : 'round'},
    INFO_VALUE_BATTERY : {ATTR_PATH : 'bat/value', ATTR_POS : 3111},
    #INFO_VALUE_TILT : {ATTR_PATH : 'accel/tilt'}, #Todo
    #INFO_VALUE_VIBRATION : {ATTR_PATH : 'accel/vibration'}, #Todo
    #INFO_VALUE_TEMP : {ATTR_PATH : 'tmp/tC'},
    #INFO_VALUE_ILLUMINANCE : {ATTR_PATH : 'lux/value'}, #Todo
    INFO_VALUE_GAS : {ATTR_PATH : 'concentration/ppm'},
    INFO_VALUE_SENSOR : {ATTR_PATH : 'gas_sensor/sensor_state'},
    INFO_VALUE_TOTAL_WORK_TIME : {ATTR_PATH : 'total_work_time'},
}

SHELLY_TYPES = {
    'SHSW-1': {'name': "Shelly 1", 'mqtt':'shelly1'},
    'SHSW-21': {'name': "Shelly 2", 'mqtt':'shellyswitch'},
    'SHSW-22': {'name': "Shelly HD Pro"},
    'SHSW-25': {'name': "Shelly 2.5", 'mqtt':'shellyswitch25'},
    'SHSW-44': {'name': "Shelly 4 Pro", 'mqtt':'shelly4pro'},
    'SHPLG-1': {'name': "Shelly Plug", 'mqtt':'shellyplug'},
    'SHPLG2-1': {'name': "Shelly Plug"},
    'SHPLG-S': {'name': "Shelly Plug S", 'mqtt':'shellyplug-s'},
    'SHRGBWW-01': {'name': "Shelly RGBWW"},
    'SHBLB-1': {'name': "Shelly Bulb", 'mqtt':'shellybulb'},
    'SHCB-1': {'name': "Shelly Bulb"},
    'SHHT-1': {'name': "Shelly H&T", 'battery' : True, 'mqtt':'shellyht'},
    'SHRGBW2': {'name': "Shelly RGBW2", 'mqtt':'shellyrgbw2'},
    'SHEM': {'name': "Shelly EM", 'mqtt':'shellyem'},
    'SHEM-3': {'name': "Shelly 3EM", 'mqtt':'shellyem3'},
    'SHCL-255': {'name': "Shelly Bulb"},
    'SH2LED-1': {'name': "Shelly 2LED"},
    'SHSK-1': {'name': "Shelly Socket"},
    'SHSW-PM': {'name': "Shelly 1 PM", 'mqtt':'shelly1pm'},
    'SHWT-1': {'name': "Shelly Flood", 'battery' : True, 'mqtt':'shellyflood'},
    'SHDM-1': {'name': "Shelly Dimmer", 'mqtt':'shellydimmer'},
    'SHDM-2': {'name': "Shelly Dimmer 2", 'mqtt':'shellydimmer2'},
    'SHDW-1': {'name': "Shelly Door/Window", 'battery' : True,
               'mqtt':'shellydw'},
    'SHDW-2': {'name': "Shelly Door/Window 2", 'battery' : True,
               'mqtt':'shellydw2'},
    'SHBDUO-1': {'name': "Shelly Duo", 'mqtt':'ShellyBulbDuo'},
    'SHBVIN-1': {'name': "Shelly Vintage", 'mqtt':'ShellyVintage'},
    'SHBTN-1': {'name': "Shelly Button 1", 'mqtt':'shellybutton1',
                'battery' : True},
    'SHIX3-1': {'name': "Shelly i3", 'mqtt':'shellyix3'},
    'SHGS-1': {'name': "Shelly Gas", 'mqtt':'shellygas'},
    'SHAIR-1': {'name': "Shelly Air", 'mqtt':'ShellyAir'},
    'SHPLG-U1': {'name': "Shelly Plug US", 'mqtt':'shellyplugu1'},
    'SHUNI-1': {'name': "Shelly UNI", 'mqtt':'shellyuni'},
    'SHSW-L': {'name': "Shelly 1L", 'mqtt':'shelly1l'}
}

EFFECTS_RGBW2 = [
    {'name': "Off", 'effect': 0},
    {'name': "Meteor shower", 'effect': 1},
    {'name': "Gradual change", 'effect': 2},
    {'name': "Flash", 'effect': 3}
]

EFFECTS_BULB = [
    {'name': "Off", 'effect': 0},
    {'name': "Meteor shower", 'effect': 1},
    {'name': "Gradual change", 'effect': 2},
    {'name': "Breath", 'effect': 3},
    {'name': "Flash", 'effect': 4},
    {'name': "On/off gradual", 'effect': 5},
    {'name': "Red/green change", 'effect': 6},
]

RSSI_LEVELS = {
    -50 : "excelent",
    -60 : "very good",
    -70 : "good",
    -1000: "weak"
}



import json
import re
from datetime import timedelta

from .compat import s
from .utils import exception_log
from .compat import urlopen
from .const import (
    REGEX_VER
)
from .loop import Loop

class Firmware_manager(Loop):

    def __init__(self, parent):
        super(Firmware_manager, self).__init__("FirmwareManage", parent,
                                    timedelta(minutes=10))
        self.parent = parent
        self.list = {}
        self.last_updated = None
        self.start_loop()

    def loop_timer(self):
        self.list = self._http_get('https://repo.shelly.cloud/files/firmware')

    def _http_get(self, url):
        f = None
        try:
            f = urlopen(url)
            body = f.read()
            res = json.loads(s(body))
            return res['data']
        except Exception as ex:
            exception_log(ex, "Error http GET: {}", url)
        finally:
            if f:
                f.close()
        return {}

    def format(self, value):
        ver = re.search(REGEX_VER, value)
        if ver:
            return ver.group(2) # + " (" + ver.group(1) + ")"
        return value

    def version(self, shelly_type, beta):
        if shelly_type in self.list:
            cfg = self.list[shelly_type]
            if beta and 'beta_ver' in cfg:
                return self.format(cfg['beta_ver'])
            else:
                return self.format(cfg['version'])

    def url(self, shelly_type, beta):
        if shelly_type in self.list:
            cfg = self.list[shelly_type]
            if beta and 'beta_ver' in cfg:
                return cfg['beta_url']
            else:
                return None
